import general.Tracer;
import general.utils.Utils;

import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.analyzer.record.ARecord4NiosII;

import app.Application;
import parsers.Parser;
import parsers.finddecl.ParseException;

/** classe ProvaParsingTotale
 * @author pierfrancesco
 *	Questa e' la classe di partenza da lanciare per selezionare
 *	con la grafica la cartella dei sorgenti e partire con l'analisi
 *  PARTIRE DA QUI - CLASSE DI PARTENZA
 */


public class Lauterbach extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	/**
	 * @param args
	 * @throws IOException 
	 * @throws Trace32Exception 
	 * @throws ParseException 
	 */
	public static void main(String[] args) throws IOException, Trace32Exception, ParseException {
		Tracer.init();		// sul vero T32 non va passato niente come parametro
		new Lauterbach();
	
	}
	
	
	public  Lauterbach() throws IOException, Trace32Exception, ParseException {
		
		String nome_dir="";	// nome della directory iniziale da cui partire 
		String executable=""; // nome del file eseguibile
		String file_xml="";
		// finestra principale
		this.setSize(500,300);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		// titolo della finestra
		this.setTitle("Parsing per Lauterbach!");
		JPanel panel = new JPanel();
		panel.show();
		this.setVisible(true);
		
		// dialogo per scegliere la CARTELLA DEI SORGENTI
		this.getContentPane().add(panel);
		JFileChooser file_chos=new JFileChooser();
		file_chos.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);	//scelgo solo cartelle
		file_chos.setDialogTitle("Scegli la cartella dei sorgenti...");
		int result=file_chos.showOpenDialog(panel);
		if (result==JFileChooser.APPROVE_OPTION) {
			File file=file_chos.getSelectedFile();
			nome_dir=file.getCanonicalPath();
		}
		
		//dialogo per scegliere il FILE ESEGUIBILE
		this.getContentPane().add(panel);
		file_chos=new JFileChooser(nome_dir);	//parto dalla cartella con i sorgenti
		file_chos.setFileSelectionMode(JFileChooser.FILES_ONLY);	//scelgo solo file
		file_chos.setDialogTitle("Scegli il file eseguibile...");
		result=file_chos.showOpenDialog(panel);
		if (result==JFileChooser.APPROVE_OPTION) {
			File file=file_chos.getSelectedFile();
			executable=file.getCanonicalPath();
		}
//		dialogo per scegliere il FILE XML
		this.getContentPane().add(panel);
		file_chos=new JFileChooser(nome_dir);	//parto dalla cartella con i sorgenti
		file_chos.setFileSelectionMode(JFileChooser.FILES_ONLY);	//scelgo solo file
		file_chos.setDialogTitle("Salva il file XML...");
		result=file_chos.showSaveDialog(panel);
		if (result==JFileChooser.APPROVE_OPTION) {
			File file=file_chos.getSelectedFile();
			file_xml=file.getCanonicalPath();
		}
		Application app=new Application(executable);  //creo l'applicazione
		// faccio il parsing di tutti i sorgenti
		Parser p=new Parser(app, nome_dir,executable, panel);
		app.fill();
		app.printFunctions();
		app.printTask();
		//adesso leggo i tempi
		long max=Tracer.t32.AnaStatusMax();
		long min=Tracer.t32.AnaStatusMin();
		
		// faccio partire il cronometro
		long inizio=System.currentTimeMillis();
		// leggo tutto il buffer di debug
		app.leggiBuffer(panel);
		// fermo il cronometro
		long fine=System.currentTimeMillis();
		//calcolo il tempo impiegato
		long tempo_ms=fine-inizio;
		//calcolo il tempo impiegato per ciclo
		double tempo_p_ciclo=((double) tempo_ms)/(-min+1);
		//stampo i risultati
		System.out.print("TEMPO NECESSARIO PER IL CICLO COMPLETO: ");
		System.out.println(tempo_ms+ "ms");
		System.out.println("TEMPO PER OGNI CICLO: "+tempo_p_ciclo+"ms");
		System.out.println("PER "+(-min+1)+" CICLI");
		app.xml(file_xml);
		XMLTree t=new XMLTree(file_xml);
		JScrollPane scroll=new JScrollPane(t);
		panel.add(scroll);
		this.getContentPane().add(t);
		t.setVisible(true);
		t.show(true);
		//termino il collegamento con il tracer
		Tracer.exit();
		
		
	}	
	
}
