import general.Tracer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import block.codeblock.BlockCode;
import block.codeblock.task.Task;
import block.datablock.BlockData;
import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.analyzer.record.ARecord4NiosII;


/* Classe Principale: Prova per vedere cosa legge*/
/* velocita': */
/* 1652ms per 14967 cicli */
/* TEMPO PER OGNI CICLO: 0.11037616088728536ms */
/* se i calcoli non sono sbagliati circa 9060 record al secondo */
/* per 1Milione di record ci vogliono circa 110 sec */

/* Classe per provare il calcolo dei tempi
*/
public class ProvaCompute {

	
	public static void main(String[] args) throws Trace32Exception, IOException {
		// inizializzo il tracer
		Tracer.init();
				
		// prendo il minimo e il max degli indici dei record nell'analyzer
		long max=Tracer.t32.AnaStatusMax();
		long min=Tracer.t32.AnaStatusMin();
		
		ARecord4NiosII r;
		
		// creo un nuovo task
		Task t=new Task(1,"main_task",0x00,0x2000);
		// lo inizializzo (aggiungendo i vari blocchi)
		// nome del file di configurazione
		// (e' del tipo <type> <symbol_name>
		// tipo e' o CODE o DATA
		// symbol_name e' il nome del simbolo
		String file_conf="C:\\config.laut";
		inizializza(t, file_conf);
		
		long inizio=System.currentTimeMillis();
		// ciclo per leggere tutti i record
		for(long num_record= min;num_record<=max;num_record++) {
			//System.out.println("Leggo record n."+num_record);
			r=new ARecord4NiosII(num_record);
			//long tim=r.getTime_exec();
			//System.out.println("Il record "+num_record+" ha eseguito per "+tim+" clock"  );
			t.leggiRecord(r);
			//magic.leggiRecord(r);
		}
		long fine=System.currentTimeMillis();
		long tempo_ms=fine-inizio;
		double tempo_p_ciclo=((double) tempo_ms)/(-min+1);
		System.out.print("TEMPO NECESSARIO PER IL CICLO COMPLETO: ");
		System.out.println(tempo_ms+ "ms");
		System.out.println("TEMPO PER OGNI CICLO: "+tempo_p_ciclo+"ms");
		System.out.println("PER "+(-min+1)+" CICLI");
		// stampo il report
		t.printReport();

		
		Tracer.exit();
	}
	
	
	public static void inizializza(Task t, String nomefile) throws IOException, Trace32Exception {
		
		// leggo il file di configurazione (per il momento)
		// per inizializzare i vari blocchi
		File f = new File(nomefile);
		BufferedReader config = new BufferedReader(
				new FileReader(f));
		int count_line=0;
		String line = config.readLine();
		while (line != null){
			count_line++;
			String[] data=line.split("\t");
			// tipo di simbolo
			String type=data[0].toUpperCase();
			// nome del simbolo
			String symbol=data[1];
			
			long start_addr=Tracer.t32.getSymbolAddress(symbol);
			if (start_addr==-1)
				System.err.println("ERRORE: il simbolo "+symbol+" (nella linea "+count_line+") NON ESISTE!");
			int len=Tracer.t32.getSymbolLen(symbol);
			
			if (type.compareTo("CODE")==0) { // se e' blocco di codice
				long end_addr=start_addr+len-1;
				// lo allineo ai 32 bit
				long resto= end_addr % 4;
				end_addr = end_addr-resto;
				
				BlockCode b=new BlockCode(symbol, start_addr, end_addr);
				t.addCalledFunction(b);
			}
			else {
				long end_addr=start_addr+len-1;
				BlockData b=new BlockData(symbol, start_addr, end_addr);
				t.addDataBlock(b);
			}
			
			line = config.readLine();
		}
		
		// alla fine chiudo il file
		config.close();
		BlockCode b=new BlockCode("no_where", 0x00, 0x59C);
		t.addCalledFunction(b);
		
	
	}
	
	
}
