import general.Tracer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import block.codeblock.BlockCode;
import block.codeblock.task.Task;
import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.analyzer.record.ARecord4NiosII;

public class ProvaComputeNested {

	public static void main(String[] args) throws Trace32Exception, IOException {
		// inizializzo il tracer
		Tracer.init();
				
		// prendo il minimo e il max degli indici dei record nell'analyzer
		long max=Tracer.t32.AnaStatusMax();
		long min=Tracer.t32.AnaStatusMin();
		long size=Tracer.t32.AnaStatusSize();
		ARecord4NiosII r;
		
		// creo un nuovo task
		Task t=new Task(1,"main_task",null, 0x00,0x2000);
		// lo inizializzo (aggiungendo i vari blocchi)
		// nome del file di configurazione
		
		String file_conf="C:\\nested.laut";
		inizializza(t, file_conf);
		System.out.println(size);
		long inizio=System.currentTimeMillis();
		// ciclo per leggere tutti i record
		for(long num_record= min;num_record<=max;num_record++) {
			//System.out.println("Leggo record n."+num_record);
			r=new ARecord4NiosII(num_record);
			//long tim=r.getTime_exec();
			//System.out.println("Il record "+num_record+" ha eseguito per "+tim+" clock"  );
			t.leggiRecord(r);
			//magic.leggiRecord(r);
		}
		long fine=System.currentTimeMillis();
		long tempo_ms=fine-inizio;
		double tempo_p_ciclo=((double) tempo_ms)/(-min+1);
		System.out.print("TEMPO NECESSARIO PER IL CICLO COMPLETO: ");
		System.out.println(tempo_ms+ "ms");
		System.out.println("TEMPO PER OGNI CICLO: "+tempo_p_ciclo+"ms");
		System.out.println("PER "+(-min+1)+" CICLI");
		// stampo il report
		t.printReport();

		
		Tracer.exit();
	}
	
	
	public static void inizializza(Task t, String nomefile) throws IOException, Trace32Exception {
		
		// leggo il file di configurazione (per il momento)
		// per inizializzare i vari blocchi
		File f = new File(nomefile);
		BufferedReader config = new BufferedReader(
				new FileReader(f));
		int count_line=0;
		String line = config.readLine();
		String nome="";
		while (line != null){
			count_line++;
			String[] data=line.split("\t");
			// tipo di simbolo
			String start_line=data[0];
			nome=start_line;
			String symbol="\\taskc\\"+start_line;
			long start_addr=Tracer.t32.getSymbolAddress(symbol);
			// nome del simbolo
			String end_line=data[1];
			nome= nome+"_"+end_line;
			symbol="\\taskc\\"+end_line;
			long end_addr=Tracer.t32.getSymbolAddress(symbol);
			
			
			if (start_addr==-1)
				System.err.println("ERRORE: il simbolo "+symbol+" (nella linea "+count_line+") NON ESISTE!");
			
			// lo allineo ai 32 bit
			long resto= start_addr % 4;
			start_addr = start_addr-resto;
			resto= end_addr % 4;
			end_addr = end_addr-resto;
				
			BlockCode b=new BlockCode(symbol, start_addr, end_addr);
			t.addCalledFunction(b);
						
			line = config.readLine();
		}
		
		// alla fine chiudo il file
		config.close();
		
	}

}
