package analyzer.record;

/** Rappresenta un record dell'Analyzer
 * @author Pierfrancesco
 *
 */
public  abstract class ARecord {
	
	protected long		n_record;
	protected byte 		access_class;
	protected long 		paddress;
	protected byte[]	bus_data;
	protected long 		bus_data_long;
	protected int		bus_data_width;
	protected byte		access_cycle;
	protected byte[] 	status_lines;
	protected long		timestamp;
	protected long		time_exec;
	protected byte[]	external_trigger;
	protected byte		logical_access_class;
	protected long		laddress;
	
	
	
	/** Costruttore di default
	 * 
	 */
	public ARecord() {
		this.bus_data=new byte[8];
		this.status_lines=new byte[2];
		this.external_trigger=new byte[2];
		
	}
	
	/** Costruttore
	 * @param nrecord			num del record del tracer da analizzare
	 * 
	 * 
	 */
	public ARecord(long nrecord)  {
		this();
		this.n_record=nrecord;

	}
	
	
	public long getNumRecord() {return n_record;}
	public long getPAddress() {return paddress;}
	public long getLAddress() {return laddress;}
	public long getTimestamp() {return timestamp;}
	
	public int getBusWidth() {return bus_data_width;}
	public long getBusDatalong() {return bus_data_long;}
	public byte getAccessClass() {return access_class;}
	public byte getAccessCycle() {return access_cycle;}
	public byte getLogicalAccessClass() {return logical_access_class;}
	
	public byte[] getBusData() {
		byte[] b=new byte[8];
		System.arraycopy(this.bus_data,0,b,0,8);
		return b;
	}
	
	public byte[] getStatusLines() {
		byte[] b=new byte[2];
		System.arraycopy(this.status_lines,0,b,0,8);
		return b;
	}
	
	public byte[] getExternalTrigger() {
		byte[] b=new byte[2];
		System.arraycopy(this.external_trigger,0,b,0,8);
		return b;
	}
	
	/**
	 * @param access_class The access_class to set.
	 */
	public void setAccess_class(byte access_class) {
		this.access_class = access_class;
	}
	
	/**
	 * @param access_cycle The access_cycle to set.
	 */
	public void setAccess_cycle(byte access_cycle) {
		this.access_cycle = access_cycle;
	}
	
	/**
	 * @param bus_data The bus_data to set.
	 */
	public void setBus_data(byte[] bus_data) {
		System.arraycopy(bus_data,0, this.bus_data,0,8);
	}
	
	/**
	 * @param bus_data_long The bus_data_long to set.
	 */
	public void setBus_data_long(long bus_data_long) {
		this.bus_data_long = bus_data_long;
	}
	
	/**
	 * @param bus_data_width The bus_data_width to set.
	 */
	public void setBus_data_width(int bus_data_width) {
		this.bus_data_width = bus_data_width;
	}
	
	/**
	 * @param external_trigger The external_trigger to set.
	 */
	public void setExternal_trigger(byte[] external_trigger) {
		System.arraycopy(external_trigger,0,this.external_trigger,0,2);
	}
	
	/**
	 * @param laddress The laddress to set.
	 */
	public void setLaddress(long laddress) {
		this.laddress = laddress;
	}
	
	/**
	 * @param logical_access_class The logical_access_class to set.
	 */
	public void setLogical_access_class(byte logical_access_class) {
		this.logical_access_class = logical_access_class;
	}
	
	/**
	 * @param n_record The n_record to set.
	 */
	public void setN_record(long n_record) {
		this.n_record = n_record;
	}
	
	/**
	 * @param paddress The paddress to set.
	 */
	public void setPaddress(long paddress) {
		this.paddress = paddress;
	}
	
	/**
	 * @param status_lines The status_lines to set.
	 */
	public void setStatus_lines(byte[] status_lines) {
		System.arraycopy(status_lines,0,this.status_lines,0,2);
		
	}
	
	
	/**
	 * @param timestamp The timestamp to set.
	 */
	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}
	
	/**
	 * @return Returns the time_exec.
	 */
	public long getTime_exec() {
		return time_exec;
	}
	
	/**
	 * @param time_exec The time_exec to set.
	 */
	public void setTime_exec(long time_exec) {
		this.time_exec = time_exec;
	}
	
	public abstract boolean isReadAccessCycle();
	
	public abstract boolean isWriteAccessCycle();
	
	public abstract boolean isDataAccessClass();
	
	public abstract boolean isProgramAccessClass();
	
	public abstract ARecord next();
	
	public abstract boolean isJump(); 
	
	public abstract boolean isCondJump();
	
	public abstract long getDestJump();

	public abstract boolean isCall();
	
	public abstract boolean isRet();

	public abstract int getDestCall();
}	

