//commentata
package app;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import javax.swing.ProgressMonitor;

import exception.OutOfSequenceException;
import general.Tracer;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;
import cpu.nios2.analyzer.record.ARecord4NiosII;

import analyzer.record.ARecord;
import block.Block;
import block.codeblock.BlockCode;
import block.codeblock.set.CodeBlocks;
import block.codeblock.set.Tasks;
import block.codeblock.task.Task;

/**
 * @author Pierfrancesco
 *
 */
public class Application {
	
	private Tasks 			set_of_task;
	private CodeBlocks		functions;
	private CodeBlocks		cregion;
	private Task			last_task;
	private String			name;		//nome dell'applicazione
	
	// relativi alla gestione degli errori
	private long			next_ip;
	private Vector			out_sequence;
	private	long			curr_n_record;
	
	/** Costruttore di default
	 * 
	 */
	public Application()  {
		
		this.set_of_task=new Tasks();
		this.functions=new CodeBlocks();
		this.cregion=new CodeBlocks();
		this.last_task=null;
		this.name="";
		
		this.next_ip=-1;
		this.curr_n_record=0;
		this.out_sequence=new Vector();

	}
	
	public Application(String nomeapp)  {
		
		this.set_of_task=new Tasks();
		this.functions=new CodeBlocks();
		this.cregion=new CodeBlocks();
		this.last_task=null;
		this.name=nomeapp;
		
		this.next_ip=-1;
		this.curr_n_record=0;
		this.out_sequence=new Vector();

	}
	
	
	
	/** Aggiunge un task t all'insieme dei tasks
	 * @param t		task da aggiungere
	 */
	public void addTask(Block t) {
		this.set_of_task.add(t);
	}
	
	public void printTask() {
		System.out.println("TASKS DELL'APPLICAZIONE");
		this.set_of_task.printElements();
		
	}
	
	/** Restituisce il task di nome name (se esiste)
	 * @param name		nome del task
	 * @return
	 */
	public Task getTask(String name) {
		return this.set_of_task.isTask(name);
	}
	
	/** Aggiunge una funzione all'applicazione
	 * @param b		blocco della funzione da aggiungere
	 */
	public void addFunction(BlockCode b) {
		this.functions.add(b);
	}
	
	/** Aggiunge una regione critica all'applicazione
	 * @param cr	blocco della ragione critica da aggiungere
	 */
	public void addCriticalRegion(BlockCode cr) {
		this.cregion.add(cr);
	}
	
	/** Restituisce una funzione con un dato nome
	 * @param name	nome della funzione da restituire
	 * @return
	 */
	public BlockCode getFunction(String name) {
		int indice =this.functions.get(name);
		return (BlockCode) this.functions.get(indice);
	}
	
	/** Legge il buffer del tracer per prendere le info
	 * @param panel
	 * @throws Trace32Exception
	 */
	public void leggiBuffer(Component panel) throws Trace32Exception{
		
		long max=Tracer.t32.AnaStatusMax(); //-1 (record piu' attuale, quello al quale si finisce)
		long min=Tracer.t32.AnaStatusMin(); // record piu' vecchio, quello dal quale si comincia)
		this.curr_n_record=min;
		ARecord4NiosII r;
		ProgressMonitor monitor=new ProgressMonitor(panel,"Sto leggendo i record...","Lettura record dalla traccia",(int) min, (int) max);
		while(this.curr_n_record<=max) {
			System.out.println("Leggo record n."+this.curr_n_record);
			// controllo se c'e' context switch
			r=new ARecord4NiosII(this.curr_n_record);
			try {
				check_sequence(r);
			} catch (OutOfSequenceException e) {
				System.err.println("Si e' verificato un errore nella sequenza!");
				// inserisco nell'elenco il numero di record dopo il quale
				// si e' verificato il fuori-sequenza
				Long obj_n_record=new Long(this.curr_n_record);
				this.out_sequence.add(obj_n_record);
				
				// provo a recuperare l'errore del fuori sequenza
				recover_sequence();
			}
			monitor.setProgress((int) this.curr_n_record);
			this.leggiRecord(r);
			this.curr_n_record++;
		}
		monitor.close();
		
	}
	
	public void leggiRecord(ARecord r) {
		if (r.isDataAccessClass())
			this.functions.leggiData(r);
		else
			this.functions.leggiRecord(r);
		
		
	}
	
	

	
	/** Controlla se ci siano buchi
	 * @param r
	 * @throws OutOfSequenceException
	 */
	private void check_sequence(ARecord r) throws OutOfSequenceException {
		
/*		if (this.next_ip==r.getPAddress()) {
			
			if ( (!r.isJump()) && (!r.isCall()) ) { // se non e' un salto (qualunque) o una call
				this.next_ip=r.getPAddress()+Nios2CPU.lengthInstr();
			}
			else {
				if (r.isCall())		// se e' una call
					this.next_ip=r.getDestCall();
				if (r.isJump())		// se e' un jump (qualunque)
					this.next_ip=r.getDestJump();
				
			}
			
		}
		else
			throw new OutOfSequenceException();
		*/
		
	}
	
	public void fill() throws Trace32Exception {
		
		this.functions.fill();
	}
	
	public void printFunctions() {
		this.functions.printElements();
	}
	
	public void xml(String nomefile) throws IOException {
		File file_xml=new File(nomefile);
		PrintWriter file_text=new PrintWriter(
				new BufferedWriter(
						new FileWriter(file_xml,false)));
		
		// scrivo il tag iniziale dell'XML
		file_text.println("<?xml version=\"1.0\"  encoding=\"UTF-8\"?>");
		// scrivo il css relativo
		file_text.println("<?xml-stylesheet type=\"text/css\" href=\"report.css\"?>");
		
		file_text.println("<APPLICATION>");
		file_text.println("\t<NAME>" +this.name +"</NAME>");

		this.set_of_task.xml(2,file_text);

		file_text.println("\t<FUNCTIONS>");
		this.functions.xml(2, file_text);
		file_text.println("\t</FUNCTIONS>");
		
		file_text.println("</APPLICATION>");
		
		//chiudo il file
		file_text.flush();
		file_text.close();
		
	}
	
	
	/** Cerca di recuperare dall'errore del fuori sequenza
	 * @throws Trace32Exception 
	 * 
	 */
	private void recover_sequence() throws Trace32Exception {
		long max=Tracer.t32.AnaStatusMax();
		
		// cancello l'attributo "interessato" per evitare
		// problemi per le call ecc...ecc...
		int n_functions=this.functions.size();
		BlockCode b;
		
		for(int j=0;j<n_functions;j++) {
			b=(BlockCode) this.functions.get(j);
			b.resetCallStack();
		}
		
		
		ARecord4NiosII rec;
		Task t=null;
		
		int n_tasks=this.set_of_task.size();
		while (this.curr_n_record<=max) {
			boolean trovato=false;
			// scorro i record fino a che non sia parte di un task!
			rec=new ARecord4NiosII(this.curr_n_record);
			// cerco se e' nel corpo di un task
			for(int i=0;i<n_tasks;i++) {
				t=(Task) this.set_of_task.get(i);
				if (t.isInMyAddressSpace(rec.getPAddress())) {
					trovato=true;
					break;
				}
			}
			if (trovato) {
				this.last_task=t;
				break;
			}
			else
				this.curr_n_record++;
		}
		
		
	}
	
	// questo per il momento non serve.
	private  void inizializza_linee(Task t, String nomefile) throws IOException, Trace32Exception {
		
		// leggo il file di configurazione (per il momento)
		// per inizializzare i vari blocchi
		File f = new File(nomefile);
		BufferedReader config = new BufferedReader(
				new FileReader(f));
		int count_line=0;
		boolean crit_section=false;
		String line = config.readLine();
		count_line++;
		String f_char=line.substring(0,2);
		// salto i commenti (se ci sono)
		while (f_char.compareTo("//")==0) {
			line = config.readLine();
			count_line++;
			f_char=line.substring(0,2);
		}
		// ciclo principale
		while (line != null){
			
			String[] data=line.split("\t");
			// tipo di simbolo
			String type=data[0].toUpperCase();
			// nome del simbolo
			String symbol=data[1];
			
			if (type.compareTo("CODE")==0) { // se e' blocco di codice
				long start_line=Long.parseLong(data[2]);
				// lo allineo ai 32 bit
				long end_line= Long.parseLong(data[3]);
				
				
				BlockCode b=new BlockCode(symbol, null, start_line, end_line);
				// lo inserisco nel vettore di funzioni
				this.functions.add(b);
				// lo inserisco nelle funzioni chiamate
				t.addCalledFunction(b);
				
				if (data.length>4) { // se ci sono sub_blocchi
					int k=4;
					while ( (k<data.length) && (data[k].compareTo("CALL")!=0)) {
						if (data[k].compareTo("CRITSEC")==0) { // se e' una sezione critica
							crit_section=true;
							k++;
							
						}
						String sub_block_name=data[k];
						String	sub_start=data[k+1];
						String	sub_end=data[k+2];
						start_line=Long.parseLong(sub_start);
						end_line=Long.parseLong(sub_end);
						BlockCode b2;
						b2=new BlockCode(sub_block_name, null,  start_line, end_line);
						b.addSubBlock(b2);
						if (crit_section)
							this.cregion.add(b2);
						k +=3;
					}
					k++;
					int max=this.functions.size();
					while (k<data.length) { // da qui ci sono le funzioni chiamate
						// cerco all'interno del vettore delle  funzioni definite quella che mi sterve
						for(int j=0;j<max;j++) {
							BlockCode func=(BlockCode) this.functions.get(j);
							if (data[k].compareTo(func.getName())==0) {
								t.addCalledFunction(func);
								break;
							}
						} // for
						k++;	
					} // while
					
					
				}
				// riempio i "buchi" eventualmente lasciati fuori
				b.fill_holes();
				
			}
			
			
			
			line = config.readLine();
			count_line++;
			
			// controlli per i commenti (inutili)
			String first_char="";
			if (line!=null) {
				first_char=line.substring(0,2);
			}
			while (first_char.compareTo("//")==0) {
				line = config.readLine();
				count_line++;
				first_char=line.substring(0,2);
			}
		}
		
		// alla fine chiudo il file
		config.close();
		
		
		// aggiungo i sub_blocchi
		
	}

	public void substFunction(BlockCode block) {
		this.functions.substitute(block);
		
	}
	
	
	
}
