// commentata
package block;

import java.io.PrintWriter;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;

import general.Tracer;
import general.utils.Utils;
import analyzer.record.ARecord;

/** Rappresenta un blocco (codice o dati) di programma
 * @author Pierfrancesco
 *
 */
public abstract  class Block {
	
	//	info relative al blocco vero e proprio
	protected String	name;			// nome del blocco
	protected boolean 	src_available;	// se il sorgente e' disponibile
	protected String	src_file;		// nome del file sorgente contenente il blocco
	protected long		start_line;		// linea di partenza del blocco se sorgente disponibile
	protected long 		end_line;		// linea finale del blocco se sorgente disponibile
	protected long		start_addr;		// indirizzo di partenza
	protected long		end_addr;		// indirizzo finale
	
	private final String NO_NAME_BLOCK="no_named_block";
	
	/** Costruttore di default
	 * 
	 */
	public Block() {
		this.name=NO_NAME_BLOCK;
		this.start_line=0;
		this.end_line=0;
		this.src_file="";
		this.src_available=false;
		this.start_addr=0;
		this.end_addr=0;
	}
	
	/** Costruttore per un blocco avendo gli indirizzi
	 * @param start_addr	indirizzo iniziale del blocco
	 * @param end_addr		indirizzo finale del blocco
	 * @throws Trace32Exception
	 */
	public  Block(long start_addr, long end_addr) throws Trace32Exception {
		this.name=NO_NAME_BLOCK;
		//prendo il nome del file in base all'indirizzo iniziale
		this.src_file=Tracer.t32.getSourceFile(start_addr);
		// il sorgente e' sempre disponibile (ipotesi)
		this.src_available=true;
		this.start_addr=start_addr;
		this.end_addr=end_addr;
		this.start_line=Tracer.t32.getSourceLine(start_addr);
		this.end_line=Tracer.t32.getSourceLine(end_addr);
	
	}
	
	
	 // se uso i numeri di linea DEVO AVERE il nome del file
	
	/** Costruttore per un blocco avendo il numero di linea
	 * @param nome_file			nome del file sorgente
	 * @param start_line		linea di inizio del blocco
	 * @param end_line			linea di fine del blocco
	 * @throws Trace32Exception 
	 */
	public Block(String nome_file,long start_line, long end_line) throws Trace32Exception {
		this.name=NO_NAME_BLOCK;
		this.start_line = start_line;
		this.end_line = end_line;
		if (nome_file.endsWith(".laut")) {
			this.src_file=nome_file.substring(0,nome_file.length()-5);
		} else
			this.src_file=nome_file;
		this.src_available=true;
		
		// aggiorno gli indirizzi
		try {
			this.start_addr=Utils.getSourceLineAddress(this.src_file, this.start_line);
		} catch (Trace32Exception e) {
			e.printStackTrace();
		}
		try {
			// allineo ai 32 bit precedenti
			this.end_addr=Utils.getSourceLineAddress(this.src_file, this.end_line+1)-Nios2CPU.lengthInstr();
			if (this.end_addr<=0)
				this.end_addr=Utils.getSourceLineAddress(this.src_file, this.end_line);
			
		} catch (Trace32Exception e) {
			e.printStackTrace();
		}
		
		if (this.start_addr==-1)
			this.start_addr=0;
		if (this.end_addr==-1)
			this.end_addr=0;
		if (this.src_file!="")
			this.src_available=true;
		else
			this.src_available=false;
		
	}
	
	
	/** Costruttore
	 * @param name				nome del blocco
	 * @param nome_file			nome del file sorgente
	 * @param start_line		linea di inizio del blocco
	 * @param end_line			linea di fine del blocco
	 * @throws Trace32Exception 
	 */
	public Block(String name,String nome_file, long start_line, long end_line) throws Trace32Exception {
		this(nome_file, start_line, end_line);
		this.name=name;
		
	}
	
	
	
	
	/** Costruttore
	 * @param name
	 * @param start_line
	 * @param end_line
	 * @param start_addr
	 * @param end_addr
	 * @throws Trace32Exception 
	 */
	public Block(String name, long start_line, long end_line, long start_addr, long end_addr) throws Trace32Exception {
		this(name,start_line, end_line);
		this.start_addr = start_addr;
		this.end_addr = end_addr;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return Returns the end_line.
	 */
	public long getEnd_line() {
		return end_line;
	}
	
	/**
	 * @param end_line The end_line to set.
	 */
	public void setEnd_line(long end_l) {
		this.end_line = end_l;
	}
	
	/**
	 * @return Returns the start_line.
	 */
	public long getStart_line() {
		return start_line;
	}
	
	/**
	 * @param start_line The start_line to set.
	 */
	public void setStart_line(long start_l) {
		this.start_line = start_l;
	}
	
	
	
	
	/**
	 * @return Returns the end_addr.
	 */
	public long getEnd_addr() {
		return end_addr;
	}

	/**
	 * @return Returns the start_addr.
	 */
	public long getStart_addr() {
		return start_addr;
	}

	/**
	 * @param end_addr The end_addr to set.
	 */
	public void setEnd_addr(long end_addr) {
		this.end_addr = end_addr;
	}

	/**
	 * @param start_addr The start_addr to set.
	 */
	public void setStart_addr(long start_addr) {
		this.start_addr = start_addr;
	}

	/**
	 * @param src_file The src_file to set.
	 */
	public void setSrc_file(String src_file) {
		this.src_file = src_file;
		this.src_available=true;
	}

	/**
	 * @return Returns the src_file.
	 */
	public String getSrc_file() {
		return src_file;
	}

	
	/**
	 * @return Returns the src_available.
	 */
	public boolean isSrc_available() {
		return src_available;
	}

	/** Verifica se l'indirizzo passato  all'interno del blocco
	 * @param addr
	 * @return 			vero se l'indirizzo passato e' all'interno del blocco 
	 * @throws Trace32Exception 
	 */
	public boolean isInMyAddressSpace(long addr)  {
		/*if (this.src_available) { // se il sorgente e' disponibile
			String namefile="";
			try { // prendo il nome del sorgente
				namefile = Tracer.t32.getSourceFile(addr); 
			} catch (Trace32Exception e) {
				e.printStackTrace();
			}
			long line=0;
			try { // prendo il numero di linea
				line = Tracer.t32.getSourceLine(addr);
			} catch (Trace32Exception e) {
				e.printStackTrace();
			}
			
			if ( (line>=this.start_line) && (line<=this.end_line) && (namefile.compareTo(this.src_file)==0))
				return true;
			else
				return false;
		} else {				*/// se il sorgente NON e' disponibile
			if ( (addr>=this.start_addr) && (addr<=this.end_addr) )
				return true;
			else
				return false;
	/*	}*/
	
	}
	
	
	
	/** Stampa un piccolo report dei dati statistici del blocco
	 * 
	 */
	public abstract void printReport();
	
	
	/** Legge un record dell'analyzer
	 * @param r			record da analizzare
	 */
	public abstract  void leggiRecord(ARecord r);
		
	
	
	/** Stampa il file XML dei risultati
	 * @param n			tabs di identazione
	 */
	public  void xml(int n, PrintWriter file) {
		String t=Utils.tabs(n);

		file.println(t+"<NAME>"+this.name+"</NAME>");
		if (this.src_available) {
			file.println(t+"\t<SRC_FILE>"+this.src_file+"</SRC_FILE>");
			file.println(t+"\t<START_LINE>"+this.start_line+"</START_LINE>");
			file.println(t+"\t<END_LINE>"+this.end_line+"</END_LINE>");
		}
		file.println(t+"\t<START_ADDR>0x"+Long.toHexString(this.start_addr)+"</START_ADDR>");
		file.println(t+"\t<END_ADDR>0x"+Long.toHexString(this.end_addr)+"</END_ADDR>");

	}
	
	
	/** Stampa la destcrizione dei blocchi per DOT
	 * @param file		file della descrizione
	 */
	public abstract void dot(PrintWriter file);
	
	
}
