// commentata
package block.codeblock;

import general.utils.Utils;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Stack;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;

import block.Block;
import block.codeblock.set.CodeBlocks;
import block.codeblock.set.SetOfPieceOfCode;
import block.codeblock.set.SubBlocks;

import analyzer.record.ARecord;
//

/** Rappresenta un blocco di codice che va computato
 * @author Pierfrancesco
 *
 */
public  class BlockCode extends PieceOfCode{
	
	
	protected SetOfPieceOfCode 	called_functions;	// vettore delle funzioni chiamate dal blocco
	protected SetOfPieceOfCode	sub_blocks;			//un vettore di sub-blocchi
	protected boolean 		interessato;		// se e' interessato per una call
												//	 o una lettura dati
	
	private int num_call;						// numero di call attive
	private Stack call_stack;					// stack delle call
											
	/** Costruttore di default
	 * @throws IOException 
	 * @throws Trace32Exception 
	 * 
	 */
	public BlockCode() {
		//chiama il costruttore della classe superiore
		super();
		// per aggiornare i sub-blocchi
		this.sub_blocks=new SubBlocks();
		this.called_functions=new CodeBlocks();
		this.interessato=false;
		this.num_call=0;
		this.call_stack=new Stack();
	}
	
	/** Costruttore avendo gli indirizzi
	 * @param start			inizio del blocco
	 * @param end			fine del blocco
	 * @throws Trace32Exception 
	 * @throws IOException 
	 * @throws Trace32Exception 
	 */
	public BlockCode(long start, long end) throws Trace32Exception{
		//chiama il costruttore della classe superiore
		super(start, end);
		// per aggiornare i sub-blocchi
		this.sub_blocks=new SubBlocks();
		this.called_functions=new CodeBlocks();
		this.interessato=false;
		this.num_call=0;
		this.call_stack=new Stack();
		
	}
	
	/** Costruttore avendo i numeri di linea
	 * @param name				nome del blocco
	 * @param nome_file			nome del file sorgente
	 * @param start				linea di inizio del blocco
	 * @param end				linea di fine del blocco
	 * @throws Trace32Exception 
	 * @throws IOException 
	 * @throws Trace32Exception 
	 */
	public BlockCode(String name, String nome_file, long start, long end) throws Trace32Exception  {
		//chiama il costruttore della classe superiore
		super(name, nome_file, start, end);
		//per aggiornare i sub-blocchi
		this.sub_blocks=new SubBlocks();
		this.called_functions=new CodeBlocks();
		this.interessato=false;
		this.num_call=0;
		this.call_stack=new Stack();
	}
	/** Costruttore avendo i numeri di linea
	 * @param name				nome del blocco
	 * @param start				linea di inizio del blocco
	 * @param end				linea di fine del blocco
	 * @throws Trace32Exception 
	 * @throws IOException 
	 * @throws Trace32Exception 
	 */
	public BlockCode(String name, long start, long end) throws Trace32Exception  {
		//chiama il costruttore della classe superiore
		super(name, "", start, end);
		//per aggiornare i sub-blocchi
		this.sub_blocks=new SubBlocks();
		this.called_functions=new CodeBlocks();
		this.interessato=false;
		this.num_call=0;
		this.call_stack=new Stack();
	}
	
	
	
	/** Aggiunge una funzione chiamata
	 * @param b		blocco di codice da aggiungere
	 */
	public void addCalledFunction(BlockCode b) {
		this.called_functions.add(b);
	}

	/* (non-Javadoc)
	 * @see block.Block#leggiRecord(analyzer.record.ARecord)
	 */
	public void leggiRecord(ARecord r) {
		if ( (this.isInMyAddressSpace(r.getPAddress())) || this.interessato ){
			//System.out.println("("+r.getNumRecord()+") P: "+this.getName());
			if (r.isCall()) {	// se leggo una call
				this.num_call++;
				this.interessato=true;
				this.call_stack.push(this);
			}
			
			if (r.isRet()) { // se leggo una ret
				this.num_call--;
				if (!this.call_stack.isEmpty())
					this.call_stack.pop();
				if (this.num_call==0)
					this.interessato=false;
			}
			
			super.leggiRecord(r);
			
			if (this.sub_blocks.size()!=0) // se ci sono sub_blocks
			// poi passo il record ai sub-blocchi
			this.sub_blocks.leggiRecord(r);
		}
		
	}
	
	/* (non-Javadoc)
	 * @see block.Block#printReport()
	 */
	public void printReport() {
		// DEBUG: per i sub-blocchi
		// stampo prima il report della classe superiore
		super.printReport();
		if (this.sub_blocks.size()>0) {
			System.out.println(">>>>>>>>>>>>>>>>>>>>>>>");
			this.sub_blocks.printReport();
			System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>");
		}
		
	}
	
	
	/** Aggiunge un sub blocco
	 * @param b			sub blocco da aggiungere
	 */
	public void addSubBlock(BlockCode b) {
		// NOTA: Se invece prima c'e' il blocco piu' interno e poi quello esterno????
		// che faccio??? Il parser deve garantire che prima dia i blocchi piu'
		// esterni e poi quelli piu' interni
		// NOTA: questo sembra gia' essere garantito dal parser
		
		BlockCode curr_block=null;
		int max=this.sub_blocks.size();
		// se non ci sono sub_blocks lo aggiungo diretamente
		if (max==0)
			this.sub_blocks.add(b);
		else { // se ci sono gia' dei sub_blocks
			boolean trovato=false;
			int i;
			for(i=0;i<max;i++) {
				curr_block=(BlockCode) this.sub_blocks.get(i);
				// se il blocco b e' interamente contenuto nel blocco corrente
				if ( (b.getStart_line()>=curr_block.getStart_line()) && (b.getEnd_line()<=curr_block.getEnd_line()) ) {
					trovato=true;
					break;
				} //if
			} // for
			if (trovato)
				curr_block.addSubBlock(b);
			else  // se non c'e' un blocco che possa contenerlo
				// allora lo aggiungo alla lista
				this.sub_blocks.add(b);
			
		} //else 
	}//method
	
	
	/* (non-Javadoc)
	 * @see block.codeblock.PieceOfCode#leggiData(analyzer.record.ARecord)
	 */
	public void leggiData(ARecord r) {
			super.leggiData(r);
			// lo propago nei sub_blocks
			this.sub_blocks.leggiData(r);
			

	}
		
	/** Riempie con blocchi gli spazi rimasti vuoti e che l'utente non ha
	 *  espressamente considerato 
	 * @throws Trace32Exception 
	 *  
	 */
	public void fill_holes() throws Trace32Exception {
		
		if (this.sub_blocks.size()>0)
			//this.sub_blocks.fill();
			;
		
	}

	/** 
	 * @param b
	 */
	public void setInteressato(boolean b) {
		this.interessato=b;

	}

	/**
	 * @return		true se ha effettuato una call
	 */
	public boolean isInteressato() {
		return this.interessato;
	}
	
	public void xml(int n, PrintWriter file) {
		String t=Utils.tabs(n);
		file.println(t+"<CODEBLOCK>");
		super.xml(n,file);
		file.println(t+"\t<CALLED_FUNC>");
		this.called_functions.xml(n+1,file);
		file.println(t+"\t</CALLED_FUNC>");
		this.sub_blocks.xml(n+2,file);
		file.println(t+"</CODEBLOCK>");

	}
	
	public void dtd(PrintWriter file) {
		file.println("<!ELEMENT CodeBlock (Name, Addr, Time, Stat, Sub_blocks)");
		file.println("<!ELEMENT Name (#PCDATA)");
		file.println("<!ELEMENT Addr ((Src_file, Start_line, End_line)?, Start_addr, End_addr)");
		file.println("<!ELEMENT Src_file (#PCDATA)");
		file.println("<!ELEMENT Start_line (#PCDATA)");
		file.println("<!ELEMENT End_line (#PCDATA)");
		file.println("<!ELEMENT Start_addr (#PCDATA)");
		file.println("<!ELEMENT End_addr (#PCDATA)");
		file.println("<!ELEMENT Time (#PCDATA)");
		this.stat.dtd(file);
		this.sub_blocks.dtd(file);
	}
	
	
	public boolean isInCalledFunc(String name) {
		return this.called_functions.isIn(name);
		
	}
	
	public void callStack(){
		Block b;
		Stack temp_stack=(Stack) this.call_stack.clone();
		while (!temp_stack.isEmpty()) {
			b=(Block) temp_stack.pop();
			System.out.print(" > "+b.getName());
		}
		
	}

	
	public void resetCallStack() {
		this.interessato=false;
		this.num_call=0;
		int max=this.called_functions.size();
		BlockCode b;
		for(int i=0;i<max;i++) {
			b=(BlockCode) this.called_functions.get(i);
			b.setInteressato(false);
			b.resetCallStack();
		}
		
	}
}
