//commentata

package block.codeblock;
import general.utils.Utils;

import java.io.PrintWriter;
import java.text.NumberFormat;
import java.util.Vector;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.CPU;
import cpu.nios2.Nios2CPU;

import block.Block;

import analyzer.record.ARecord;

import statistic.Statistics;


/** Rappresenta un blocco di CODICE
 * @author Pierfrancesco
 *
 */

public  class PieceOfCode extends Block{
	
	
	protected Vector	time_executions;// vettore dei tempi di esecuzione
	protected long 		time_exec;		// tempo di esecuzione corrente del blocco (corrente)
	protected int		curr_address;	// indirizzo attuale
	protected long		start_timestamp;
	protected long		end_timestamp;
	
	protected Statistics stat;			// statistiche
	
	
	
	/** Costruttore di default
	 * 
	 */
	public PieceOfCode() {
		super();
		this.time_executions = new Vector();
		this.stat=new Statistics();
		this.time_exec=0;
		this.curr_address=0;
		this.start_timestamp=0;
		this.end_timestamp=0;
		
	}
	
	/** Costruttore di un pezzo di codice avendo gli indirizzi
	 * @param start		indirizzo di inizio del blocco di codice
	 * @param end		indirizzo di fine del blocco di codice
	 * @throws Trace32Exception 
	 */
	public PieceOfCode(long start, long end) throws Trace32Exception {
		//chiama il costruttore di default
		super(start, end);
		this.time_executions = new Vector();
		this.stat=new Statistics();
		this.time_exec=0;
		this.curr_address=0;
		this.start_timestamp=0;
		this.end_timestamp=0;
		
		
	}
	
	
	/** Costruttore di un pezzo di codice avendo i numeri di linea
	 * @param name				nome del blocco
	 * @param nome_file			nome del file sorgente
	 * @param start_line		linea di inizio del blocco di codice
	 * @param end_line			linea di fine del blocco di codice
	 * @throws Trace32Exception 
	 */
	public PieceOfCode(String name, String nome_file, long start, long end) throws Trace32Exception {
		super(name, nome_file, start, end);
		this.time_executions = new Vector();
		this.stat=new Statistics();
		this.time_exec=0;
		this.curr_address=0;
		this.start_timestamp=0;
		this.end_timestamp=0;
	}
	
		
	
	/**
	 * @return Returns the time_executions.
	 */
	public long getTime_executions(int i) {
		Object t=new Object();
		t=this.time_executions.get(i);
		return ((Long) t).longValue();
	}
	
	/**
	 * @param time_executions The time_executions to set.
	 */
	public void addTime_executions(long time_click) {
		Long t=new Long(time_click);
		this.time_executions.add(t);
	}
	
	
	
	/** Stampa un piccolo report sul tempo di esecuzione
	 * 
	 */
	public void printReport() {
		final int MAX_FRAC=3;	// numero massimo di decimali
		
		NumberFormat nf=NumberFormat.getNumberInstance();
		nf.setMaximumFractionDigits(MAX_FRAC);
		
		System.out.println("\t  + Blocco di codice: "+this.name);
		if (this.src_available)
			System.out.println("\t    da linea: "+this.start_line+" a linea: "+this.end_line);
		else
			System.out.println("\t    da 0x"+Long.toHexString(this.start_line)+" a 0x"+Long.toHexString(this.end_line));
		System.out.print("\t    Tempo di esecuzione corr: "+nf.format(this.time_exec)+" =");
		double time_us=((this.time_exec/256D)*20D)/1000D;
		System.out.println(nf.format(time_us)+"us.");
		System.out.println("\t    STATISTICHE:");
		this.stat.compute(this.time_executions, this.time_exec);
		this.stat.printReport();
		System.out.println("  ----------------------------------------");
	}
	
	public void xml(int n, PrintWriter file) {
		String t=Utils.tabs(n);
		double tempo=Nios2CPU.to_us(this.time_exec);
		super.xml(n,file);
		file.println(t+"\t<TIME_EXEC>"+tempo+"us"+"</TIME_EXEC>");
		this.stat.compute(this.time_executions, this.time_exec);
		this.stat.xml(n+1, file);
		
	}//method
	
	public void dtd(PrintWriter file) {
		file.println("<!ELEMENT CodeBlock (Name, Addr, Time, Stat)");
		file.println("<!ELEMENT Name (#PCDATA)");
		file.println("<!ELEMENT Addr ((Src_file, Start_line, End_line)?, Start_addr, End_addr)");
		file.println("<!ELEMENT Src_file (#PCDATA)");
		file.println("<!ELEMENT Start_line (#PCDATA)");
		file.println("<!ELEMENT End_line (#PCDATA)");
		file.println("<!ELEMENT Start_addr (#PCDATA)");
		file.println("<!ELEMENT End_addr (#PCDATA)");
		file.println("<!ELEMENT Time (#PCDATA)");
		this.stat.dtd(file);
	}
	
	
	public void dot(PrintWriter file) {
		final String CR="\\l";
		long start=this.getStart_line();
		long end=this.getEnd_line();
		file.print(this.getName()+" [shape=box, label=\" ");
		file.print(this.getName()+CR+"start_line: "+start+CR+"end_line: "+end+CR);
		file.print("Total exec: "+this.getTotalExec()+" us" );
		file.print(CR+"Max: "+this.getMax()+" us");
		file.print(CR+"Min: "+this.getMin()+" us");
		file.print(CR+"Avg: "+this.getAverage()+" us");
		file.print(CR+"num exec: "+this.getNum_exec()+" times");
		file.println(" \"] ;" );	
		file.println(" { rank=same; \""+this.getName()+"\" ; }");
		
	}
	
	
	/** Legge un record dell'analyzer
	 * @param r
	 */
	public void leggiRecord(ARecord r) {
		
		// prendo l'indirizzo corrente
		long curr_address= r.getPAddress();
		
		if (curr_address==this.start_line) {
			this.start_timestamp=r.getTimestamp();
			//System.out.println("Record dove comincia l'esecuzione di "+this.name+": "+r.getNumRecord());
		}
		// sommo al tempo totale di esecuzione
		// il tempo di esecuzione del record corrente
		this.time_exec +=r.getTime_exec();
		
		// se sono arrivato alla fine del blocco
		if (curr_address==this.end_line) {
			this.end_timestamp=r.getTimestamp();

			//System.out.println("Record dove finisce l'esecuzione di "+this.name+": "+r.getNumRecord());
			//calcolo da differenza tra timestamp di inizio e fine del blocco
			//long tempo_=this.end_timestamp-this.start_timestamp;
			this.start_timestamp=0;
			this.end_timestamp=0;
			//aggiungo al vettore dei tempi il tempo corrente
			this.time_executions.add(new Long(this.time_exec));
			// e riazzero il tempo di esecuzione
			this.time_exec=0;
		} // if
		
		
	}//method
	
	
	public long getTotalExec() {
		long sum= 0;
		for(int i=0;i<this.time_executions.size();i++) {
			Long t= (Long) this.time_executions.get(i);
			sum += t.longValue();
		}
		sum +=this.time_exec;
		return sum;
	}
	
	public void leggiData(ARecord r) {
		// sommo al tempo totale di esecuzione
		// il tempo di esecuzione del record corrente
		if (r.isDataAccessClass()) 
			this.time_exec +=r.getTime_exec();
		
	}
	
	
	
	public double getMax() {	return Nios2CPU.to_us(this.stat.getMax());}
	
	public double getMin() {	return  Nios2CPU.to_us(this.stat.getMin());}
	
	public double getAverage() { return Nios2CPU.to_us(this.stat.getAverage());}
	
	public long getNum_exec() { return ( this.stat.getNum_exec()); }
	
	public double getTotal_exec() { return Nios2CPU.to_us(this.stat.getTotal_exec()); }
	
}//class
