// commentata
/**
 * 
 */
package block.codeblock.task;

import java.io.IOException;
import java.io.PrintWriter;

//import com.lauterbach.trace32.api.Trace32API;
import com.lauterbach.trace32.api.Trace32Exception;

import general.utils.Utils;
import block.codeblock.BlockCode;
import block.codeblock.set.CodeBlocks;
import block.datablock.BlockData;
import block.datablock.set.DataBlocks;
import analyzer.record.ARecord;

/**	Rappresenta un task a cui passare le info
 *  dei record letti fino a quel momento
 * @author Pierfrancesco
 *
 */
public class Task extends BlockCode{
	
	private final String	NO_NAME_TASK = "no_name"; 
	protected int			task_number;		// numero del task
	
	/* NOTA:  i blocchi di codice non possono essere innestati */
	
	protected DataBlocks	data_blocks;		// vettore delle strutture dati da monitorare
	private int counter_data;
	
	/** Costuttore di default
	 * 
	 */
	public Task()  {
		// chiamo prima il costruttore della classe superiore
		super();
		this.task_number=-1;
		this.name=NO_NAME_TASK;
		
		this.called_functions=new CodeBlocks();
		this.counter_data=0;
		this.data_blocks=new DataBlocks();
		
	}
	
	/** Costruttore
	 * @param task_number	numero del task
	 * @param name			nome del task
	 * @param nome_file		nome del file sorgente
	 * @param start			linea di inzio del task
	 * @param end			linea di fine del task
	 * @throws IOException 
	 */
	public Task(int task_number, String task_name, String nome_file, long start, long end) throws IOException, Trace32Exception {
		// chiamo prima il costruttore di default
		super(task_name, nome_file, start, end);
		this.task_number=task_number;
		this.counter_data=0;
		this.called_functions=new CodeBlocks();
		this.data_blocks=new DataBlocks();
	}
	
	/** Costruttore
	 * @param task_number	numero del task
	 * @param name			nome del task
	 * @param start			linea di inzio del task
	 * @param end			linea di fine del task
	 * @throws IOException 
	 */
	public Task(int task_number, String task_name, int start, int end) throws IOException, Trace32Exception {
		// chiamo prima il costruttore di default
		super(task_name, "", start, end);
		this.task_number=task_number;
		this.counter_data=0;
		this.called_functions=new CodeBlocks();
		this.data_blocks=new DataBlocks();
	}
	
		
	/**
	 * @return Returns the task_number.
	 */
	public int getTask_number() {
		return task_number;
	}
	
	/**
	 * @param task_number The task_number to set.
	 */
	public void setTask_number(int task_number) {
		this.task_number = task_number;
	}
	
	
	
	
	/** Aggiunge un blocco di dati da monitorare 
	 * @param b		blocco di dati da aggiungere
	 */
	public void addDataBlock(BlockData b) {
		this.data_blocks.add(b);
	}
	
	
	/** Legge un record dell'analizzatore 
	 * @param r		Record passatogli per essere esaminato
	 */
	public void leggiRecord(ARecord r) {
		
		super.leggiRecord(r);
		 /*if (r.isDataAccessClass()) {
		 this.data_blocks.leggiRecord(r);
		 this.called_functions.leggiData(r);
		 
		 counter_data++;
		 }*/
		/*else */
		// poi passo il record ai blocchi
		if (r.isDataAccessClass())
			this.called_functions.leggiData(r);
		else
			this.called_functions.leggiRecord(r); 
		
		
		
	}
	
	
	
	
	/* (non-Javadoc)
	 * @see block.PieceOfCode#printReport()
	 */
	public void printReport() {
		
		System.out.println("======== TASK N. "+this.task_number+" == "+this.name+" ========");
		super.printReport();
		System.out.println(" *********** BLOCCHI DI CODICE ************");
		this.called_functions.printReport();
		System.out.println(" *********** BLOCCHI DI DATI ************");
		this.data_blocks.printReport();
		System.out.println("NUMERO DI ACCESSI AI DATI: "+this.counter_data);
		System.out.println("===========================================");
	}
	
	/* (non-Javadoc)
	 * @see block.Block#xml(int, java.io.PrintWriter)
	 */
	public void xml(int n, PrintWriter file) {
		String t=Utils.tabs(n);
		file.println(t+"<TASK>");
		
		file.println(t+"\t<NAME>"+this.name+"</NAME>");
		file.println(t+"\t<NUMBER>"+this.task_number+"</NUMBER>");
		super.xml(n+1, file);
		file.println(t+"</TASK>");
	}
	
	
	public void dtd(PrintWriter file) {
		file.println("<!ELEMENT Task (Name, Addr, Time, Stat)");
		super.dtd(file);
	}
	

}
