// commentata
/**
 * 
 */
package block.datablock;

import java.io.PrintWriter;

import com.lauterbach.trace32.api.Trace32Exception;

import general.utils.Utils;
import analyzer.record.ARecord;
import block.Block;

/** Rappresenta un blocco di dati
 * @author Pierfrancesco
 *
 */
public abstract class PieceOfData extends Block{
	
		
	protected long num_access;	// numero di accessi alla struttura
	protected long num_write;	// numero di scritture
	protected long num_read;	// numero di letture
	
	/** Costruttore di default
	 * 
	 */
	public PieceOfData() {
		super();
		this.num_access=0;
		this.num_write=0;
		this.num_read=0;
	}
	
	
	public PieceOfData(long start, long end) throws Trace32Exception {
		super(start, end);
		this.num_access=0;
		this.num_write=0;
		this.num_read=0;
	}
	
	public PieceOfData(String name, long start, long end) throws Trace32Exception {
		this(start, end);
		this.name=name;
		
	}


	
	/**
	 * @return Returns the num_access.
	 */
	public long getNum_access() {
		return num_access;
	}
	
	

	/**
	 * @return Returns the num_read.
	 */
	public long getNum_read() {
		return num_read;
	}


	/**
	 * @return Returns the num_write.
	 */
	public long getNum_write() {
		return num_write;
	}


	/* (non-Javadoc)
	 * @see block.Block#printReport()
	 */
	public void printReport() {
		System.out.println("\t   + Blocco di dati "+ this.name);
		System.out.println("\t       da 0x"+Long.toHexString(this.getStart_line())+" a 0x"+Long.toHexString(this.getEnd_line()));
		System.out.println("\t       NUM. TOTALE DI ACCESSI: "+this.num_access);
		System.out.println("\t       Num. di scritture: "+this.num_write);
		System.out.println("\t       Num. di letture: "+this.num_read);
	}
	
	/* (non-Javadoc)
	 * @see block.Block#xml(int)
	 */
	public void xml(int n,PrintWriter file) {
		String t=Utils.tabs(n);
		file.println(t+"<DATA_BLOCK>");
		file.println(t+"\t<NAME>"+this.name+"</NAME>");
		file.println(t+"\t<START_ADDR>0x"+Long.toHexString(this.start_line)+"</START_ADDR>");
		file.println(t+"\t<END_ADDR>0x"+Long.toHexString(this.end_line)+"</END_ADDR>");
		file.println(t+"\t<NUM_ACCESS>"+this.num_access+"</NUM_ACCESS>");
		file.println(t+"\t<NUM_WRITE>"+this.num_write+"</NUM_WRITE>");
		file.println(t+"\t<NUM_READ>"+this.num_read+"</NUM_READ>");
		file.println(t+"</DATA_BLOCK>");
	}


	/* (non-Javadoc)
	 * @see block.Block#LeggiRecord(analyzer.record.ARecord)
	 */
	public void leggiRecord(ARecord r) {
		// se e' nel mio spazio di indirizzamento e l'accesso e' di tipo data (==1)
		if ( (isInMyAddressSpace(r.getPAddress())) && (r.isDataAccessClass()==true) ) {
			// allora aggiorno il num di accessi
			this.num_access++;
			if (r.isReadAccessCycle())
				this.num_read++;
			if (r.isWriteAccessCycle())
				this.num_write++;
		}
	}


	public void dot(PrintWriter file) {
		System.out.println("PieceOfData.dot(PrintWriter file)");
		file.println("PieceOfData.dot(PrintWriter file)");
		
	}
	
	
}
