//commentata

/**
 * 
 */
package block.set;

import general.Tracer;
import general.utils.UniqueNumber;
import java.io.PrintWriter;
import java.util.Vector;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;
import analyzer.record.ARecord;
import block.Block;
import block.codeblock.BlockCode;
import block.codeblock.PieceOfCode;

/**
 * @author Pierfrancesco
 *
 */
public abstract class SetOfPieces {
	
	protected Vector 	vettore;
	
	/** Costruttore di default
	 * 
	 */
	public SetOfPieces() {
		this.vettore = new Vector();
	}
	
	/** Aggiunge un blocco 
	 * @param b
	 */
	public void add(Block b) {
		if (b==null)
			return;
		if (!this.vettore.isEmpty()) {
			/*int indice=0;
			Block p=(Block) this.vettore.get(indice);
			long inizio= b.getStart_addr();	//prendo l'indirizzo di partenza
			while ((indice<this.vettore.size()) && (p.getStart_addr()<=inizio)  ) {
				p=(Block) this.vettore.get(indice);
				
				indice++;			//scorro gli indici fino a che trovo
									//un blocco che sta subito dopo
			}
			if ((p.getStart_addr()==b.getStart_addr()) && (p.getEnd_addr()==b.getEnd_addr()) )
				return;
			this.vettore.add(indice,b);*/
			
			int indice=this.get(b.getName()); // vedo se c'e' un blocco con quel nome
			if (indice!=-1)	{	// se c'e' allora  lo ignoro
				return;
			}
			this.vettore.add(b);	//altrimenti lo inserisco
			
		}
		else	//altrimenti lo inserisco all'inizio
			this.vettore.add(b);
		
		
}
	
	public void substitute (BlockCode block) {
		int indice=this.get(block.getName()); // vedo se c'e' un blocco con quel nome
		if (indice!=-1)	{	// se c'e' allora  lo sostituisco
			this.vettore.set(indice,block);
			return;
		}
		this.vettore.add(block);	//altrimenti lo inserisco
		
		
	}
	
	
	/** Stampa un piccolo report con linee di start_line ed end_line
	 *  e tempo di esecuzione
	 * 
	 */
	public void printReport() {
		Block t=null;
		for(int i=0;i<this.vettore.size();i++) {
			t=(Block) this.vettore.get(i);
			t.printReport();
		}
		
	}
	
	
	public void printElements() {
		for(int i=0;i<this.vettore.size();i++) {
			BlockCode b=(BlockCode) this.vettore.get(i);
			System.out.println("Elem n. "+i+"\t"+b.getName());
			
		}
		
		
	}
	/** Scrive sul file codice XML per la rappresentazione della classe
	 * @param n
	 * @param file
	 */
	public abstract void xml(int n, PrintWriter file);
	
	public abstract void dtd(PrintWriter file);
	
	/** Legge un record dell'analyzer
	 * @param r			record da analizzare
	 */
	
	public  void leggiRecord(ARecord r) {
		
		int i=0;
		boolean trovato=false;
		Block t=null;
		
		// prendo l'indirizzo corrente del record passato al metodo
		long addr=r.getPAddress();
		
		// cerco a quale struttura si riferisce
		for(i=0;i<this.vettore.size();i++) {
			t=(Block) this.vettore.get(i);
			if  ( (t.getStart_addr()<=addr) && (addr<=t.getEnd_addr()) ){
				trovato=true;
				break;
			}
		}
		
		// i e' il numero d'ordine del task
		// e t punta ad esso
		if (trovato) // se il blocco era stato aggiunto 
			t.leggiRecord(r);
		
	}
	
	/** Restituisce la prima occorrenza del blocco chiamanto "name"
	 * @param name		nome del blocco da cercare
	 * @return			blocco richiesto (se esiste), altrimenti null
	 */
	public int get(String name) {
		Block b=null;
		int i=0;
		boolean trovato=false;
		int max=this.vettore.size();
		for(i=0;i<max;i++) {
			b=(Block) this.vettore.get(i);
			String nome=b.getName();
			if (nome.compareTo(name)==0) {
				trovato =true;
				break;
			}
				
		}
		if (trovato)
			return i;
		else
			return -1;
	}
	
	/** Restituisce l'i-esimo blocco
	 * @param number		numero del blocco da cercare
	 * @return				blocco richiesto (se esiste), altrimenti null
	 */
	public Block get(int number) {
		Block b=null;
		if (number<0)
			return null;
		if (number<=this.vettore.size())
			b=(Block) this.vettore.get(number);
		return b;
	}
	
	
	/** ritorna true se l'indirizzo e' contenuto nei blocchi dell'insieme
	 * @param address				indirizzo da controllare
	 * @return
	 */
	public boolean isInMyAddressSpace(long address) {
		int max=this.vettore.size();
		Block b;
		boolean trovato=false;
		for(int i=0;i<max;i++) {
			b=(Block) this.vettore.get(i);
			if (b.isInMyAddressSpace(address)) {
				trovato=true;
				break;
			}//if
		}// for
		return trovato;
		
		
	}
	
	/** Restituisce il numero dei blocchi presenti
	 * @return			numero dei blocchi presenti
	 */
	public int size() {
		if (this.vettore!=null)
			return this.vettore.size();
		else
			return 0;
		
	}
	
	/** Riempie i buchi tra un blocco e un altro
	 * @throws Trace32Exception 
	 * 
	 */
	public void fill() throws Trace32Exception {
		this.sort();
		BlockCode p=null;
		if (this.vettore.size()>0) {
			p=(BlockCode) this.vettore.get(0);
			long indirizzo_iniziale=p.getStart_addr();
			if (indirizzo_iniziale>0) {	//piazzo un blocco iniziale
				BlockCode start_filler=new BlockCode("",0,indirizzo_iniziale-Nios2CPU.lengthInstr());
				this.vettore.add(0,start_filler);
			}
			p=(BlockCode) this.vettore.lastElement();
			long indirizzo_finale=p.getEnd_addr();	// piazzo un blocco finale
			BlockCode end_filler=new BlockCode("",indirizzo_finale, 0xfffffff);
			this.vettore.add(end_filler);
						
		}
		for(int indice=0;indice<(this.vettore.size()-1);indice++) { //scorro tutti i blocchi
			p=(BlockCode) this.vettore.get(indice);		//prendo l'i-esimo blocco
			BlockCode succ=(BlockCode) this.vettore.get(indice+1);
			if ((p.getEnd_addr()+Nios2CPU.lengthInstr()) < succ.getStart_addr()) { // se c'e' un buco
				//creo un nuovo blocco
				BlockCode nuovo=new BlockCode("",p.getEnd_addr()+Nios2CPU.lengthInstr(),succ.getStart_addr()-Nios2CPU.lengthInstr());
				// e lo inserisco
				this.vettore.add(indice,nuovo);		
			}
		}
		
		
	}
	
	
	/** Riempie con blocchi gli spazi rimasti vuoti e che l'utente non ha
	 *  espressamente considerato 
	 * @throws Trace32Exception 
	 * 
	 */
	/*public void fill_holes() throws Trace32Exception {
		
		long start=Long.MAX_VALUE;
		long end=Long.MIN_VALUE;
		int max=this.vettore.size();
		Block b;
		if (this.vettore.size()>0) {
			for(int i=0;i<max;i++) {
				b=(Block) this.vettore.get(i);
				if (b.getStart_addr()<start)
					start=b.getStart_addr();
				if (b.getEnd_addr()>end)
					end=b.getEnd_addr();
			}// for
		}// if
		
		
		this.fill_holes(start,end);
		
		
	}//method
	*/
	
	/** Riempie con blocchi gli spazi rimasti vuoti e che l'utente non ha
	 *  espressamente considerato 
	 * @param start			indirizzo di partenza dello spazio da riempire
	 * @param end			indirizzo finale dello spazio da riempire
	 * @throws Trace32Exception 
	 */
/*	public void fill_holes(long start, long end) throws Trace32Exception {
	
		long counter=start;
		long start_curr_block, end_curr_block;
		Block curr_block=null;
		
		UniqueNumber unique_ident=new UniqueNumber(100);
		
		while(counter<end) {
			
			// scorro gli indirizzi gia' occupati
			while ( (this.isInMyAddressSpace(counter)) && (counter<end) )
				counter +=Nios2CPU.lengthInstr();
			// a questo punto il counter e' un indirizzo libero
			start_curr_block=counter;
			
			// scorro i liberi
			while ( (!this.isInMyAddressSpace(counter))  && (counter<end) )
				counter +=Nios2CPU.lengthInstr();
			// adesso counter e' il primo indirizzo occupato
			if (start_curr_block==end)
				return;
			
			if (counter<end)
				end_curr_block=counter-Nios2CPU.lengthInstr();
			else
				end_curr_block=counter;
			
			// posso creare il blocco
			// vedo di che tipo di blocchi e' il set
			String nome_classe="";
			String nome_blocco="block_"+unique_ident.str_number();
			String nome_file=Tracer.t32.getSourceFile(start);
			if (this.size()>0)
				nome_classe=this.get(0).getClass().getName();
			// in base al nome istanzio la classe
			if (nome_classe.compareTo("block.codeblock.PieceOfCode")==0) {
				curr_block=new PieceOfCode(nome_blocco, nome_file, start_curr_block, end_curr_block);
				curr_block.setStart_addr(start_curr_block);
				curr_block.setEnd_addr(end_curr_block);
				
				curr_block.setStart_line(-1);
				curr_block.setEnd_line(-1);
			}
			if (nome_classe.compareTo("block.codeblock.BlockCode")==0) {
				curr_block=new BlockCode(nome_blocco, nome_file, start_curr_block, end_curr_block);
				curr_block.setStart_addr(start_curr_block);
				curr_block.setEnd_addr(end_curr_block);
				
				curr_block.setStart_line(-1);
				curr_block.setEnd_line(-1);
			}
			
			// lo aggiungo alla lista
			this.add(curr_block);
			
		}// while
		
	}
	*/
	public boolean isIn(String name) {
		
		int max=this.vettore.size();
		Block b;
		for(int i=0;i<max;i++) {
			b=(Block) this.vettore.get(i);
			if (b.getName().compareTo(name)==0) 
				return true;
			
		}
		return false;

	}
	
	/** Ordina il vettore in base all'indirizzo di partenza dei blocchi
	 * 
	 */
	private void sort() {
		if (this.vettore.isEmpty())  // se il vettore  vuoto 
			return;
		int max=this.vettore.size();
		int posizioneMin=0;
		BlockCode bj=null;
		BlockCode bmin=null;
		for (int i = 0; i < max-1; i++) {
		      // Ricerca del minimo
			  posizioneMin = i;
		      for (int j = i + 1; i < max; j++) {
		    	bj=(BlockCode) this.vettore.get(j);
		    	bmin=(BlockCode) this.vettore.get(posizioneMin);
		        if (bj.getStart_addr() < bmin.getStart_addr()) {
		          posizioneMin = j;
		        }
		      }
		      // Scambio del minimo con il primo
		      BlockCode t = (BlockCode) this.vettore.get(i);
		      this.vettore.set(i,bmin);
		      this.vettore.set(posizioneMin,t);
		    }

		
	}
	

}
