package com.lauterbach.trace32.api;

/**  
 * @author oli
 * 
 * Test program for Trace32API class.
 * 
 * The program is designed to be used with the Starcore simulator demo supplied
 * by Matthias Zens. It opens some T32 windows so you can single step the demo 
 * and see the effects in T32. 
 */
public class Apitest {
	
	static Trace32API t32;
	static ITrace32Listener theListener;
	
	static String fileName = null;
	
	static final int SC     = 0;  /* Starcore   */     
	static final int ZSP500 = 1;  /* LSI ZSP5xx */
	
	public static void startup() {
		try {
			
			theListener = new ITrace32Listener() {

				public void createdBreakpoint(Trace32Breakpoint bp) {
					System.out.println("Created breakpoint " + bp);
				};

				public void deletedBreakpoint(Trace32Breakpoint bp) {
					System.out.println("Deleted breakpoint " + bp);
					/*
					try {
						t32.deleteBreakpoint(bp);
					} catch (Trace32Exception e) {
						System.out.println("CAUGHT EXCEPTION: " + e);		
					}*/
					
				};

				public void modifiedBreakpoint(Trace32Breakpoint bp) {
					System.out.println("Modified breakpoint " + bp);
				};
			};

			int target = SC;
			

			switch (target) {

			case SC:
				t32 = Trace32API.getTrace32API(20002); /* my local configs */
				fileName = "demo.c";
				break;

			case ZSP500:
				t32 = Trace32API.getTrace32API(20002);
				t32.command("cd C:/var/workspace/com.lauterbach.trace32.api/demo.zsp500");
				t32.command("do demo.cmm"); // sys.cpu zsp500, load sieve.elf,
				// b.s main, b.s sieve
				fileName = "sieve.c";
				break;

			default:
				t32 = null;
			}
		}
	
			catch (Trace32Exception e) {
				// Did you configure the config.t32 startup file for JAPI?
				System.out.println(e);
			}
	}
		
	public static void main(String[] args) {

		try {
			startup();

			System.out.println("This is a rudimentary demo application for T32 JAPI.");

			/*
			 * Open the "area" window for showing text messages and checking
			 * commands.
			 */
			t32.command("area");
			t32.command("print \"Hello, T32!\"");

			/*
			 * Open break.point list window to show currently active breaks (use
			 * right mouse button to manipulate)
			 */
			t32.command("break.list");

			/* The listener will print to System.out on each invocation */
			t32.addListener(theListener);

			createPollingThreads(4);
			
			t32.enablePolling(true, 2000);

			/*
			 * Will set a number of breaks on consecutive source lines. Some of
			 * these lines belong to the same HLL line. Therefore the debugger
			 * will create fewer breakpoints than calls are made to
			 * setBreakpoint().
			 */

			for (int i = 190; i < 220; i++) {
				t32.setBreakpoint(fileName, i);
			}

			/*
			 * Dump all breakpoints to see effect. Note: You can use the
			 * "break.list" command in T32 to display a window with all breaks.
			 */
			t32.dumpAllBreakpoints();

			/*
			 * Disable all breakpoints; notifications will be printed by the
			 * listeners
			 */
			Trace32Breakpoint[] a;
			a = t32.getBreakpoints();
			for (int i = 0; i < a.length; i++) {
				a[i].enable(false);
			}

			/* Enable all breakpoints */
			a = t32.getBreakpoints();
			for (int i = 0; i < a.length; i++) {
				a[i].enable(true);
			}

			/*
			 * Delete all breakpoints. NOTE: The command "break.delete /ALL"
			 * does the same in T32.
			 */
			a = t32.getBreakpoints();
			for (int i = 0; i < a.length; i++) {
				t32.deleteBreakpoint(a[i]);
			}

			/*
			 * Will create notifications for all changes to the remote
			 * breakpoint state that are not yet known to the API. As the
			 * current implementation does not acutally read breakpoints from
			 * the debugger, will not create any notifications.
			 */
			t32.updateStatus();

			t32.exit();
			// TODO while (t32.isPracticeRunning()); // not yet working
			//synchronized (this) {wait(500);}

		} catch (Trace32Exception e) {
			// Did you configure the config.t32 startup file for JAPI?
			System.out.println(e);
		}

	}
	
	static void createPollingThreads(int number) {
		
		Thread[] t = new Thread[number];

		for (int i = 0; i < number; i++) {
			t[i] = new Thread() {
				public void run() {
					while (true) {
						try {
							t32.updateStatus();
						//	System.out.println("--Polled status--");
							sleep(500);
						} catch (InterruptedException e) {
						} catch (Trace32Exception e) {
						}
					}
				}
			};
			t[i].start();
		}
		 
	}
	
	
}

