/*
 * Listener to report (generic) state changes in T32.
 * 
 * Currently changes to the breakpoint status are implemented. Future extensions
 * may included listeners for the target state (run, stopped), debugger state
 * (up, down, ...), errors.
 * 
 * The listener methods are called independently of the source of the
 * modifications e.g. - when the debugger creates breakpoints (via GUI or
 * scripts) - the JAPI is used to modify breakpoint setting.
 */

package com.lauterbach.trace32.api;

public interface ITrace32Listener {

	/*
	 * Called when a breakpoint is created. Note that T32 merges breakpoints
	 * e.g. when two breaks are set on the same HLL line. Therefore setting a
	 * breakpoint does not necessarily trigger a notification.
	 */
	void createdBreakpoint(Trace32Breakpoint bp);

	/*
	 * Called when a breakpoint is deleted in T32. The breakpoint object should
	 * not be used after that
	 */
	void deletedBreakpoint(Trace32Breakpoint bp);

	/*
	 * Called when properties of bp changed. In some case, this will be reported
	 * as deletedBreakpoint/createdBreakpoint.
	 */
	void modifiedBreakpoint(Trace32Breakpoint bp);
	
	
	
	/*
	 
	 Future extensions:
	 
	 void changedRunState(int newState);
	 
	 void changedHeadMode(int newHeadMode);

	 */
	
}
