/** commentata
 * 
 */
package cpu.nios2.analyzer.record;


import general.Tracer;
import general.utils.Utils;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;

import analyzer.record.ARecordOnLine;

/** Rappresenta un record dell'analyzer specifico  per il NiosII
 * @author Pierfrancesco
 *
 */
public class ARecord4NiosII extends ARecordOnLine {
	
	private static final byte READ_CYCLE = 4;
	private static final byte WRITE_CYCLE = 5;
	private static final byte MASK_DATA_ACCESS=1;
	private static final byte MASK_PROG_ACCESS=2;
	
	
	/** Costruttore
	 * @param nrecord			numero del record
	 * @param t32
	 * @throws Trace32Exception
	 */
	public ARecord4NiosII(long nrecord) throws Trace32Exception {
		// richiamo il costruttore della classe superiore 
		super(nrecord);
		
		byte[] buffer= new byte[128];
		
		Tracer.t32.AnaRecordGet( (int) nrecord, buffer, 128);
		
		//aggiorno la classe di accesso fisico
		this.access_class=buffer[2];
		
		//aggiorno il physical address (little endian)
		//s="";
		//for(i=7;i>=4;i--)
		//s +=Utils.toHex(buffer[i]);
		//super.setPaddress(Long.parseLong(s,16));
		this.paddress=Utils.toIntegerLE(buffer,4);
					
		//aggiorno il bus data
		System.arraycopy(buffer,8,this.bus_data,0,8);
		//inverto l'ordine dei byte
		//for(int j=0;j<=7;j++)
		//	temp_bus2[j]=temp_bus1[7-j];
		//super.setBus_data(temp_bus2);
		
		// lo converto in long
		//s="";
		//for(i=0;i<=7;i++)
		//	s +=Utils.toHex(temp_bus2[i]);
		//super.setBus_data_long(Long.parseLong(s,16));
		this.bus_data_long=Utils.toLongLE(buffer,8);
		
		
		//aggiorno la larghezza del bus
		this.bus_data_width =buffer[16];
		
		//aggiorno il ciclo di accesso al bus
		this.access_cycle =buffer[17];
		
		//aggiorno la linea di stato
		//byte []temp_status=new byte[2];
		//System.arraycopy(buffer,18,temp_status,0,2);
		//super.setStatus_lines(temp_status);
		System.arraycopy(buffer,18,this.status_lines,0,2);
		
		//aggiorno il timestamp
		//s=""; // riazzero la stringa
		//for(i=27;i>=20;i--)
		//	s +=Utils.toHex(buffer[i]);
		//byte[] timestamp_array=new byte[8];
		//super.setTimestamp(Long.parseLong(s,16));
		this.timestamp =Utils.toLongLE(buffer, 20);;
		
		//aggiorno l' external trigger A/B inputs
		//byte [] temp_external=new byte[2];
		//System.arraycopy(buffer,28,temp_external,0,2);
		//super.setExternal_trigger(temp_external);
		System.arraycopy(buffer,28,this.external_trigger,0,2);
		
		//aggiorno il logical access class
		this.logical_access_class =buffer[30];
		
		//aggiorno il logical address
		//s="";
		//for(i=35;i>=32;i--)
		//	s +=Utils.toHex(buffer[i]);
		//super.setLaddress(Long.parseLong(s,16));
		this.laddress=Utils.toIntegerLE(buffer, 32);
		
		// calcolo il tempo di esecuzione
		this.time_exec=this.timestamp-Tracer.lastTimestamp;
		Tracer.lastTimestamp=this.timestamp;
	}
		
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isReadAccessCycle()
	 */
	public boolean isReadAccessCycle() {
		if (this.access_cycle==READ_CYCLE)
			return true;
		else	
			return false;
		
	}
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isWriteAccessCycle()
	 */
	public boolean isWriteAccessCycle() {
		if (this.access_cycle==WRITE_CYCLE)
			return true;
		else
			return false;
		
	}
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isDataAccessClass()
	 */
	public boolean isDataAccessClass() {
		
		if ((this.access_class & MASK_DATA_ACCESS)!=0)
			return true;
		else
			return false;
		
	}
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isProgramAccessClass()
	 */
	public boolean isProgramAccessClass() {
		
		if ((this.access_class & MASK_PROG_ACCESS)!=0)
			return true;
		else
			return false;
	}

	/**
	 * @return		restituisce l'opcode del jump (se e' un jump)
	 */
	private long getOpCodeJump() {
		return Nios2CPU.getOpCodeJump(this.bus_data_width, this.bus_data_long);
	}
	
	/**
	 * @return		restituisce l'istruzione corrente
	 */
	private  long getCurrInstr() {
		return Nios2CPU.getCurrInstr(this.bus_data_width, this.bus_data_long);
		
	}
	/** Controlla se il record descrive un salto (qualunque)
	 * @return				true se rappresenta un salto (qualunque)
	 */
	public boolean isJump() {
		long opcode=this.getOpCodeJump();
		return Nios2CPU.isJump(opcode);
		
	}
	
	/** Controlla se il record descrive un salto condizionato
	 * @return				true se rappresenta un salto condizionato
	 */
	
	public boolean isCondJump() {
		
		long opcode=this.getOpCodeJump();
		return Nios2CPU.isCondJump(opcode);
	}
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isRet()
	 */
	public boolean isRet() {
		if (this.isProgramAccessClass()) {
		long curr_instr=this.getCurrInstr();
		return Nios2CPU.isRet(curr_instr);
		} else
			return false;
	}
	
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#isCall()
	 */
	public boolean isCall() {
		if (this.isProgramAccessClass()) {
		long curr_instr=this.getCurrInstr();
		return Nios2CPU.isCall(curr_instr);
		} else
			return false;
		
	}
	
	/** restituisce l'indirizzo di destinazione del salto
	 * @return				indirizzo di destinazione del salto
	 */
	public long getDestJump() {
		return Nios2CPU.getDestJump(this.bus_data_width, this.bus_data_long, this.paddress);
		
	}
	
	/* (non-Javadoc)
	 * @see analyzer.record.ARecord#getDestCall()
	 */
	public int getDestCall() {
		return (int) Nios2CPU.getDestCall(this.bus_data_width, this.bus_data_long);
		
	}
	
		
	
}
