package cpu.nios2.branch;
import general.Tracer;
import general.utils.Utils;
import branch.BranchRecord;

import com.lauterbach.trace32.api.*;

import cpu.nios2.Nios2CPU;




/**Rappresenta un branch record per NiosII
 * @author Pierfrancesco
 * non serve al progetto attuale
 */
public class BranchRecord4NiosII extends BranchRecord{
//	rappresenta un oggetto branch 
	
	// ATTENZIONE: questa classe
	// funziona solo per Nios II
	
	
	private byte rA,rB,rC; // il num dei registri interessati
	
	
	public BranchRecord4NiosII(long address, int type, byte ra, byte rb, byte rc, int immediate) {
		super(address,type);
		
		rA = ra;
		rB = rb;
		rC = rc;
		
	}
	/**
	 * @param address
	 * @param type
	 * @param ra
	 * @param rb
	 */
	public BranchRecord4NiosII(long address, int type, byte ra, byte rb) {
		super(address,type);
		rA = ra;
		rB = rb;
		this.rC=0;
	}
	
	
	public BranchRecord4NiosII() {
		super();
		this.rC=0;
	}
	
	public BranchRecord4NiosII(long addr,int typ) {
		super(addr,typ);
		this.rA=0;
		this.rB=0;
		this.rC=0;
	}
	
	public long address(){
		return super.address();
	}
	
	public void getAddress(long a) {
		super.setAddress(a);
	}
	
	public int getType() {
		return super.getType();
	}
	
	public void setType(int t) {
		super.setType(t);
	}
	
	public boolean getYetExec() {
		return super.getYetExec();
	}
	
	public void setYetExec(boolean b) {
		super.setYetExec(b);
	}
	
	
	public void jump(boolean condition) {
		// se condition==true fa saltare
		// se condition==false NON fa saltare
		// agisce  modificando i registri (particolare implementazione per Nios II)
		
		int regA, regB, regC;
		int[] registers=new int[64];
		String comando="";
		
		
		long mask=0xffffffff;
		// leggo tutti i registri operandi dell'istruzione
		// cosi' mi tolgo il pensiero
		try {
			registers=Tracer.t32.readRegisters(mask);
		} catch (Trace32Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		regA=registers[rA]; // regA<=contenuto del registro A
		regB=registers[rB];
		regC=registers[rC];
		
		long urb=Utils.toUnsigned(regB);
		long ura=Utils.toUnsigned(regA);	
		// a seconda del tipo
		//System.out.print("0x"+Integer.toHexString(this.address)+" ");
		switch (getType()) {
		
		case Nios2CPU.BEQ: {		//System.out.print("BLT ");
			if (condition)
				regA = regB;
			// regA==regB e quindi SALTA
			else
				regA =regB+1;
			// regA!=regB e quindi NON SALTA
			
			
		}	break;
		
		case Nios2CPU.BGE: {	//System.out.print("BGE ");
			if (condition)
				regA = regB;
			// regA>=regB e quindi SALTA
			else
				regA =regB-1;
			// regA<regB e quindi NON SALTA
			
		} break;
		
		case Nios2CPU.BNE: {	//System.out.print("BNE ");
			if (condition) {
				regA=regB+1;
				//System.out.println(" SALTA");
				// regA e regB sono diversi
				// e quindi SALTA
			}
			else {
				regA=regB;
				//System.out.println(" NON SALTA");
				// regA e regB sono UGUALI
				// e quindi NON SALTA
			} 
		}	break;
		
		case Nios2CPU.BLT: { 	//System.out.print(" BLT");
			if (condition)
				regA = regB-1;
			// regA<regB e quindi SALTA
			else
				regA =regB+1;
			// regA>regB e quindi NON SALTA
		}	break;
		
		case Nios2CPU.BGEU: {	//System.out.print("BGEU");
			
			if (condition) {
				ura=urb+1;
				regA=Utils.toSigned(ura);
			} else { 
				urb=ura+1;
				regB=Utils.toSigned(urb);
			}
			
		}
		
		case Nios2CPU.BLTU:{
			if (condition) {
				ura=urb-1;
				regA=Utils.toSigned(ura);
			} else { 	urb=ura-1;
			regB=Utils.toSigned(urb);
			}
		}
		
		default:	;
		
		} // switch
		
		// aggiorno i registri con le modifiche
		comando="register.set R"+this.rA+" "+Integer.toHexString(regA);
		try {
			Tracer.t32.command(comando);
		} catch (Trace32Exception e) {
			
			e.printStackTrace();
		}
		
		comando="register.set R"+this.rB+" "+Integer.toHexString(regB);
		try {
			Tracer.t32.command(comando);
		} catch (Trace32Exception e1) {
			
			e1.printStackTrace();
		}
		
		comando="register.set R"+this.rC+" "+Integer.toHexString(regC);
		try {
			Tracer.t32.command(comando);
		} catch (Trace32Exception e) {
			
			e.printStackTrace();
		}
		
	}
	
	
}
