/**
 * 
 */
package general.utils;

import general.Tracer;

import java.util.Vector;

import com.lauterbach.trace32.api.Trace32Exception;

import cpu.nios2.Nios2CPU;

import block.Block;
import block.codeblock.BlockCode;
import block.codeblock.PieceOfCode;
import block.set.SetOfPieces;

/** Classe di utilit generale
 * @author Pierfrancesco
 * commentata
 */
public class Utils {
	
	
	/** Converte un byte nella sua rappresentazione in 
	 * esadecimale come stringa
	 * 
	 * @param b				byte da convertire
	 * @return
	 */
	public static String toHex(byte b) {
		int i;
		String s, s2;
		i= (short) b;
		s=Integer.toHexString(i);
		//System.out.println("Lunghezza stringa :"+s.length());
		
		if (s.length()>1)
			s2=s.substring(s.length()-2,s.length());
		else {
			s2=s.substring(s.length()-1,s.length());
			s2 = "0"+s2;
		}
		return s2.toUpperCase();
	}
	
	/** Converte un signed int in un Unsigned long
	 * @param x
	 * @return
	 */
	public static long toUnsigned(int x) {
		long maschera= 4294967295L;
		long temp=0;
		temp = x;
		temp= x & (maschera);
		
		return temp;
	}
	
	/** Converte un unsigned long in un signed int
	 * @param x
	 * @return
	 */
	public static int toSigned(long x) {
		long maschera= 4294967295L;
		long temp;
		temp=x;
		temp = temp & maschera;
		temp= (int) temp;
		
		return (int) temp; 
	}
	
	/** Crea una stringa di n caratteri di tabulazione
	 * @param n			numero dei caratteri di tabulazione
	 * @return			stringa contenente il numero di tabulazioni passato
	 */
	public static String tabs(int n) {
		StringBuffer s=new StringBuffer("");
		for(int i=1;i<=n;i++)
			s.append("\t");
		
		return  s.toString();
		
	}
	
	public static long alignto4(long x){
		int resto= (int) (x % 4);
		return (x - resto);		
		
	}
	
	/** restituisce il numero intero formato dai byte del buffer
	 *  in notazione little-endian 
	 *  (al byte di indice piu' basso  c'e' il byte meno significativo)
	 *  (per es. byte	0	1	2	3
	 *  				12	34	56	78
	 *  diventa l'intero 0x78563412)
	 *  
	 *  				
	 * @param buf		vettore di 4 byte che rappresentano l'intero
	 * @return
	 */
	public static int toIntegerLE(byte[] buf) {
		int numero =0;
		
		int s1=buf[0] & 0x000000FF;
		int s2= (buf[1] << 8) & 0x0000FF00;
		int s3=(buf[2]<<16) & 0x00FF0000;
		int s4=(buf[3]<<24) & 0xFF000000;
		
		numero = s1 | s2 | s3 | s4;
		return numero;
	}
	
	
	/** restituisce il numero intero formato dai byte del buffer
	 *  in notazione little-endian 
	 *  (al byte di indice piu' basso  c'e' il byte meno significativo)
	 *  (per es. byte	0	1	2	3
	 *  				12	34	56	78
	 *  diventa l'intero 0x78563412)
	 *  
	 *  				
	 * @param buf		vettore di byte in cui ci sono i bytes da esaminare
	 * @param			indice di partenza
	 * @return
	 */
	public static int toIntegerLE(byte[] buf, int start) {
		int numero =0;
		
		int s1=buf[start+0] & 0x000000FF;
		int s2= (buf[start+1] << 8) & 0x0000FF00;
		int s3=(buf[start+2]<<16) & 0x00FF0000;
		int s4=(buf[start+3]<<24) & 0xFF000000;
		
		numero = s1 | s2 | s3 | s4;
		return numero;
	}
	
	
	
	
	/** Restituisce il numero intero formato dai byte del buffer
	 *  in notazione big-endian 
	 *  (al byte di indice piu' basso  c'e' il byte piu' significativo)
	 *  
	 * @param buf		vettore di 4 byte che rappresentano l'intero
	 * @return
	 */
	public static int toIntegerBE(byte[] buf) {
		int numero =0;
		
		int s1=buf[3] & 0x000000FF;
		int s2= (buf[2] << 8) & 0x0000FF00;
		int s3=(buf[1]<<16) & 0x00FF0000;
		int s4=(buf[0]<<24) & 0xFF000000;
		
		numero = s1 | s2 | s3 | s4;
		return numero;
	}
	
	
	/** Restituisce il numero intero formato dai byte del buffer
	 *  in notazione big-endian 
	 *  (al byte di indice piu' basso  c'e' il byte piu' significativo)
	 *  
	 * @param buf		vettore di byte in cui ci sono i bytes da esaminare
	 * @param			indice di partenza
	 * @return
	 */
	public static int toIntegerBE(byte[] buf, int start) {
		int numero =0;
		
		int s1=buf[start+3] & 0x000000FF;
		int s2= (buf[start+2] << 8) & 0x0000FF00;
		int s3=(buf[start+1]<<16) & 0x00FF0000;
		int s4=(buf[start+0]<<24) & 0xFF000000;
		
		numero = s1 | s2 | s3 | s4;
		return numero;
	}
	
	/** Restituisce il long formato dai byte del buffer
	 *  in notazione little-endian 
	 *  (al byte di indice piu' basso  c'e' il byte meno significativo)
	 *  (per es. byte	0	1	2	3	4	5	6	7
	 *  				12	34	56	78	9A	BC	DE	F0
	 *  diventa il long 0xF0DEBC9A78563412)
	 * @param buf		vettore di 8 byte che rappresentano il long
	 * @return
	 */
	public static long toLongLE(byte[] buf) {
		long numero=0;
		long s0=buf[0] & 		0x00000000000000FFL;
		long s1=(buf[1] << 8) & 0x000000000000FF00L;
		long s2=(buf[2] <<16) & 0x0000000000FF0000L;
		long s3=(buf[3] <<24) & 0x00000000FF000000L;
		long s4=(buf[4] <<32) & 0x000000FF00000000L;
		long s5=(buf[5] <<40) & 0x0000FF0000000000L;
		long s6=(buf[6] <<48) & 0x00FF000000000000L;
		long s7=(buf[7] <<56) & 0xFF00000000000000L;
		
		numero= s0| s1|s2|s3|s4|s5|s6|s7;
		return numero;
	}
	
	/** Restituisce il long formato dai byte del buffer
	 *  in notazione little-endian 
	 *  (al byte di indice piu' basso  c'e' il byte meno significativo)
	 *  (per es. byte	0	1	2	3	4	5	6	7
	 *  				12	34	56	78	9A	BC	DE	F0
	 *  diventa il long 0xF0DEBC9A78563412)
	 * @param buf		vettore di byte in cui ci sono i bytes da esaminare
	 * @param			indice di partenza
	 * @return
	 */
	public static long toLongLE(byte[] buf,int start) {
		long numero=0;
		long s0= (long)buf[start+0] 		& 0x00000000000000FFL;// byte meno significativo
		long s1=((long)buf[start+1] << 8) & 0x000000000000FF00L;
		long s2=((long)buf[start+2] <<16) & 0x0000000000FF0000L;
		long s3=((long)buf[start+3] <<24) & 0x00000000FF000000L;
		long s4=((long)buf[start+4] <<32) & 0x000000FF00000000L;
		long s5=((long)buf[start+5] <<40) & 0x0000FF0000000000L;
		long s6=((long)buf[start+6] <<48) & 0x00FF000000000000L;
		long s7=((long)buf[start+7] <<56) & 0xFF00000000000000L;// byte piu' significativo
		
		numero= s0| s1|s2|s3|s4|s5|s6|s7;
		return numero;
	}
	
	
	
	/** Restituisce il long formato dai byte del buffer
	 *  in notazione big-endian 
	 * @param buf		vettore di 8 byte che rappresentano il long
	 * @return
	 */
	public static long toLongBE(byte[] buf) {
		long numero=0;
		long s0=buf[7] & 		0x00000000000000FFL;
		long s1=(buf[6] << 8) & 0x000000000000FF00L;
		long s2=(buf[5] <<16) & 0x0000000000FF0000L;
		long s3=(buf[4] <<24) & 0x00000000FF000000L;
		long s4=(buf[3] <<32) & 0x000000FF00000000L;
		long s5=(buf[2] <<40) & 0x0000FF0000000000L;
		long s6=(buf[1] <<48) & 0x00FF000000000000L;
		long s7=(buf[0] <<56) & 0xFF00000000000000L;
		
		numero= s0| s1|s2|s3|s4|s5|s6|s7;
		return numero;
	}
	
	/** Restituisce il long formato dai byte del buffer
	 *  in notazione big-endian 
	 * @param buf		vettore di byte in cui ci sono i bytes da esaminare
	 * @param			indice di partenza
	 * @return
	 */
	public static long toLongBE(byte[] buf, int start) {
		long numero=0;
		long s0=buf[start+7] & 		  0x00000000000000FFL;
		long s1=(buf[start+6] << 8) & 0x000000000000FF00L;
		long s2=(buf[start+5] <<16) & 0x0000000000FF0000L;
		long s3=(buf[start+4] <<24) & 0x00000000FF000000L;
		long s4=(buf[start+3] <<32) & 0x000000FF00000000L;
		long s5=(buf[start+2] <<40) & 0x0000FF0000000000L;
		long s6=(buf[start+1] <<48) & 0x00FF000000000000L;
		long s7=(buf[start+0] <<56) & 0xFF00000000000000L;
		
		numero= s0| s1|s2|s3|s4|s5|s6|s7;
		return numero;
	}
	
	/** Riempie con blocchi gli spazi rimasti vuoti e che l'utente non ha
	 *  espressamente considerato 
	 * @param start_line				indirizzo di partenza da cui riempire gli spazi
	 * @param end_line				indirizzo di partenza da cui riempire gli spazi		
	 * @param occupati			set di blocchi gia' occupati
	 * @return					set di blocchi che riempiono gli spazi rimasti
	 * @throws Trace32Exception 
	 */
	public static Vector buchi(long start, long end, SetOfPieces occupati) throws Trace32Exception {
		long counter=start;
		Vector liberi=new Vector();
		long start_curr_block, end_curr_block;
		Block curr_block=null;
		String nome_file=Tracer.t32.getSourceFile(start);
		UniqueNumber unique_ident=new UniqueNumber(100);
		
		while(counter<end) {
			
			// scorro gli indirizzi gia' occupati
			while ( (occupati.isInMyAddressSpace(counter)) && (counter<end) )
				counter +=Nios2CPU.lengthInstr();
			// a questo punto il counter e' un indirizzo libero
			start_curr_block=counter;
			
			// scorro i liberi
			while ( (!occupati.isInMyAddressSpace(counter))  && (counter<end) )
				counter +=Nios2CPU.lengthInstr();
			// adesso counter e' il primo indirizzo occupato
			if (start_curr_block==end)
				System.out.println("Il blocco e' tutto pieno!!!!!");
			
			if (counter<end)
				end_curr_block=counter-Nios2CPU.lengthInstr();
			else
				end_curr_block=counter;
			
			// posso creare il blocco
			// vedo di che tipo di blocchi e' il set
			String nome_classe="";
			String nome_blocco="block_"+unique_ident.str_number();
			
			if (occupati.size()>0)
				nome_classe=occupati.get(0).getClass().getName();
			// in base al nome istanzio la classe
			if (nome_classe.compareTo("block.codeblock.PieceOfCode")==0) {
				curr_block=new PieceOfCode(nome_blocco, nome_file, start_curr_block, end_curr_block);;
			}
			if (nome_classe.compareTo("block.codeblock.BlockCode")==0) {
				curr_block=new BlockCode(nome_blocco, nome_file, start_curr_block, end_curr_block);;
			}
			
			
			// lo aggiungo alla lista
			liberi.add(curr_block);
			
		}// while
		if (liberi.size()>0)
			return liberi;
		else
			return null;
		
	}
	
	/** Restituisce l'indirizzo di una data linea di un dato file
	 * @param nomefile		nome del file sorgente
	 * @param line			numero della linea
	 * @return				indirizzo corrispondente alla linea
	 * @throws Trace32Exception
	 */
	public static long getSourceLineAddress(String nomefile, long line) throws Trace32Exception {
		// compongo la stringa
		int first=nomefile.lastIndexOf("\\");
		int len=nomefile.length();
		String lone_name=nomefile.substring(first,len);	//solo il nome del file
		int point=lone_name.indexOf(".");
		String str_line=lone_name.substring(0,point)+"\\"+line;
		long addr=Tracer.t32.getSymbolAddress(str_line);
		return addr;

	}
	
	public static long allinea(long addr) {
		
		return Nios2CPU.allinea(addr);
	}
	
}
