//commentata


package parsers;

import general.Tracer;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import javax.swing.ProgressMonitor;

import com.lauterbach.trace32.api.Trace32Exception;

import parsers.cparser.CParser;
import parsers.finddecl.FindDeclarations;
import parsers.finddecl.ParseException;

import app.Application;

/** Effettua il parsing di tutti i files sorgenti C all'interno della cartella del progetto
 * @author Pierfrancesco
 *
 */
public class Parser {
	// estensione (suffisso) dei sorgenti in cui sono state espanse le macro
	private static final String EXT_EXPANDED_MACRO=".laut";
	private static final String EXT_SOURCE_FILES=".c";
	private static final String ELIM_DIRECTIVE="__attribute__";
	private static final String ELIM_MODE="((__mode__ (__SI__)))";
	private static final String ELIM_ALWAYS="((always_inline))";
	private static final String TEMP_LAUT="C:\\temp.temp";
	
	ElencaFiles elenco;
	Vector lista_c;					// lista dei files .c
	Vector lista_laut;				// lista dei files .laut
	Application application_ref;	// riferimento all'oggetto Applicazione
	
	
	
	/** Costruttore
	 * @param app				riferimento all'applicazione
	 * @param directory			nome della directory in cui risiedono i sorgenti del progetto
	 * @param executable		nome dell'eseguibile ELF
	 * @param panel				riferimento al componente genitore
	 * @throws IOException
	 * @throws Trace32Exception
	 * @throws ParseException 
	 */
	public Parser(Application app, String directory, String executable, Component panel) throws IOException, Trace32Exception, ParseException {
		// cerco i files sorgenti .c
		this.elenco=new ElencaFiles(directory, EXT_SOURCE_FILES);
		this.lista_c=this.elenco.getListFiles();
		// cerco i files espansi .laut
		this.elenco=new ElencaFiles(directory, EXT_EXPANDED_MACRO);
		this.lista_laut=this.elenco.getListFiles();
		this.application_ref=app;
		int n_tasks=0;
		
		Tracer.t32.command("data.load.elf "+executable); // carico l'eseguibile
		
		int max=this.lista_laut.size();
		ProgressMonitor monitor=new ProgressMonitor(panel,"Sto effettuando il parsing dei sorgenti...","",0,max);
		monitor.setMillisToPopup(0);
		

		for(int i=0;i<max;i++) {
			String nomefile=(String) this.lista_laut.get(i);
			monitor.setNote("Leggo il file "+nomefile );
			monitor.setProgress(i);
			//faccio la patch per eliminare quelle direttive per il compilatore
			// noiose che non riesco a processare con il CParser
			System.out.println("Metto la patch al file "+nomefile);
			this.patch(nomefile); 
			// II^ passata del parsing di nomefile
			// che scrive su un file tutto quello che devo istanziare
			// o meglio: chiama i costruttori appositi
			System.out.println("I FASE: trovo le funzioni!");
			FindDeclarations fd=new FindDeclarations(app,nomefile);
		}
		
		for(int i=0;i<max;i++) {
			String nomefile=(String) this.lista_laut.get(i);
			monitor.setNote("Leggo il file "+nomefile );
			monitor.setProgress(i);
			//cambio il nome del sorgente da .c (sorgenti con le macro)
			// a .c.laut (sorgenti cone le macro espanse)
			//int point= nomefile.lastIndexOf(".");
			//nomefile=nomefile.substring(0,point)+EXT_EXPANDED_MACRO;
			monitor.setNote("Leggo il file "+nomefile );
			System.out.println("II FASE: faccio il parsing dei file espansi");
			CParser wr=new CParser(app, nomefile);
		} //for
		
		FindTasks first_pass=new FindTasks();
		max=this.lista_c.size();	// numero totale dei files sorgenti da processare
		
		
		for(int i=0;i<max;i++) {
			String nomefile=(String) this.lista_c.get(i);
			monitor.setNote("Leggo il file "+nomefile );
			monitor.setProgress(i);
			// I^ passata del parsing alla ricerca dei tasks
			System.out.println("III FASE: ricerco i tasks");
			System.out.println("cerco i tasks nel file "+nomefile);
			n_tasks +=first_pass.find(nomefile, app);
			// TODO: adesso fai il preprocessing di ogni file
			// cioe' espandi tutte le macro dimodoche' la seconda passata trovi
			// solo codice C puro!
			// (ci pensa gia' il makefile che espande le macro
			// e salva il file come .laut !!!!!!!! ) (speranza)
			// mi serve solo che il makefile espanda tutte le macro di ogni file
			// e salvi il risultato come nome_file_c.laut
		}

		monitor.close();
	}
	
	/** patch dei file per eliminare parti non processabili dal CParser
	 * @param nomefile
	 * @throws IOException 
	 */
	private void patch(String nomefile) throws IOException {
		
		File f_temp=new File(TEMP_LAUT);
		File f = new File(nomefile);
		//rinomino il file .laut nel file temporaneo
		f.renameTo(f_temp);
		//lo apro e lo leggo riga per riga
		BufferedReader in=new BufferedReader(new FileReader(f_temp));
		String line=in.readLine();
		//apro un file in scrittura con un nome temporaneo
		File file_out=new File(nomefile);
		PrintWriter fout=new PrintWriter(new BufferedWriter(new FileWriter(file_out)));
		//leggo in sequenza tutte le righe del file
		int index=0;
		while (line !=null) {
			index=line.indexOf(ELIM_DIRECTIVE);
			//quando trovo la parte incriminata
			if (index !=-1) 
				line=line.substring(0,index)+line.substring(index+ELIM_DIRECTIVE.length(),line.length());
			
			index=line.indexOf(ELIM_MODE);
			//quando trovo la parte incriminata
			if (index !=-1) 
				line=line.substring(0,index)+line.substring(index+ELIM_MODE.length(),line.length());

			index=line.indexOf(ELIM_ALWAYS);
			//quando trovo la parte incriminata
			if (index !=-1) 
				line=line.substring(0,index)+line.substring(index+ELIM_ALWAYS.length(),line.length());

			//e scrivo il resto sul file chiamato come quello originario
			fout.println(line);
			line=in.readLine();
		}
		in.close();
		fout.flush();
		fout.close();

		//cancello il file temporaneo
		File f2=new File(TEMP_LAUT);
		f2.delete();
		
	}
	
	
	
	//questo per il momento non serve
	/** Costruttore
	 * @param app				riferimento all'applicazione
	 * @param directory			nome della directory in cui risiedono i sorgenti del progetto
	 * @throws IOException
	 * @throws Trace32Exception
	 * @throws ParseException 
	 */
	/*public Parser(Application app, String directory) throws IOException, Trace32Exception, ParseException {
		
		// cerco i files sorgenti .c
		this.elenco=new ElencaFiles(directory, ".c");
		this.lista_c=this.elenco.getListFiles();
		// cerco i files espansi .laut
		this.elenco=new ElencaFiles(directory, EXT_EXPANDED_MACRO);
		this.lista_laut=this.elenco.getListFiles();
		this.application_ref=app;
		
		FindTasks first_pass=new FindTasks();
		int max=this.lista_c.size();
		
		
		// I fase: cerco solo i tasks
		System.out.println("I FASE: ricerco i tasks");
		System.out.flush();
		for(int i=0;i<max;i++) {
			String nomefile=(String) this.lista_c.get(i);
			System.out.println("  Leggo il file "+nomefile );
			System.out.flush();
			// I^ passata del parsing alla ricerca dei tasks
			
			first_pass.find(nomefile, app);
		}
		// TODO: adesso fai il preprocessing di ogni file
		// cioe' espandi tutte le macro dimodoche' la seconda passata trovi
		// solo codice C puro!
		// (ci pensa gia' il makefile che espande le macro
		// e salva il file come .laut !!!!!!!! ) 
		// mi serve solo che il makefile espanda tutte le macro di ogni file
		// e salvi il risultato come nome_file_c.laut
		
		// II^ passata del parsing di nomefile
		// che scrive su un file tutto quello che devo istanziare
		// o meglio: chiama i costruttori appositi
		
		// II FASE: trovo tutte le funzioni
		max=this.lista_laut.size();
		// adesso cerco i file .laut
		System.out.println("II FASE: trovo le funzioni!");
		System.out.flush();
		for(int i=0;i<max;i++) {
			String nomefile=(String) this.lista_laut.get(i);
			System.out.println("  Leggo il file "+nomefile );
			System.out.flush();
			
		
		//FindDeclarations fd=new FindDeclarations(app,nomefile);
		
		//cambio il nome del sorgente da .c (sorgenti con le macro)
		// a .laut (sorgenti cone le macro espanse)
		
		System.out.println("Adesso faccio il parsing del file espanso "+nomefile);
		CParser wr=new CParser(app, nomefile);
		
		
		
		}
		System.out.println(" ************   FINE PARSING DEL PROGETTO ********************************");
		
	}
	*/
	
}



