package parsers.cparser;
import general.Tracer;

import com.lauterbach.trace32.api.Trace32Exception;

import block.codeblock.BlockCode;
import app.Application;


public class CreaFileConfig {
	
	
	private StackOfBlock st;					// Stack di blocchi
	private StackOfBlock called_task; 			// stack di funzioni chiamate da una funzione
	private Application	app;					// riferimento all'oggetto Applicazione
	
	/** Costruttore di default
	 *
	 */
	public CreaFileConfig(Application app) {
		
		this.st=new StackOfBlock();	
		this.called_task=new StackOfBlock();
		this.app=app;
		
	}
	
	/** Controlla se un blocco e' una funzione 
	 * @param curr_file_parsed 
	 * @throws Trace32Exception 
	 * @throws Trace32Exception 
	 *
	 */
	public void controlla(Blocco b, String nome_file, Application app) throws Trace32Exception  {
		BlockCode block;
		boolean isCritSection=false;
		if (b==null)
			return;
		if (b.getType()==Blocco.FUNCTION) { //se e' una dichiarazione di funzione
			System.out.print("CODE\t"+b.getNome());
			System.out.print("\t"+b.getLinea_inizio());
			System.out.print("\t"+b.getLinea_fine());
			
			/* PATCH per rimediare all'errore dei numeri di linea */
			long nlinea_espanso=b.getLinea_inizio();
			int indir_simbolo=Tracer.t32.getSymbolAddress(b.getNome());
			long nlinea_indirizzo=Tracer.t32.getSourceLine(indir_simbolo);
			long diff=nlinea_indirizzo-nlinea_espanso;			
			/* END PATCH */
						
			block=new BlockCode(b.getNome(), nome_file, b.getLinea_inizio()+diff, b.getLinea_fine()+diff);
		
			/*  qui ci sono i sub-blocchi */	
			while (!st.pila.empty()) {
				System.out.print("\t");
				Blocco estratto=st.pop();
				if (estratto.getType()==Blocco.CRITSECTION) {
					System.out.print("CRITSEC\t");
					isCritSection=true;
				}
				System.out.print(estratto.getNome());
				System.out.print("\t");
				System.out.print(estratto.getLinea_inizio());
				System.out.print("\t");
				System.out.print(estratto.getLinea_fine());
				// istanzio un sub_blocco
				BlockCode sub;
				sub=new BlockCode(estratto.getNome(), nome_file, estratto.getLinea_inizio()+diff, estratto.getLinea_fine()+diff);
				// e lo aggiungo alla funzione di cui fa parte
				block.addSubBlock(sub);
				if (isCritSection) {
					isCritSection=false;
					this.app.addCriticalRegion(sub);
				}
				
			}

			/* qui ci sono le funzioni chiamate */
			/*if (!called_task.pila.empty()) {	// se chiama una qualche funzione
				System.out.print("\tCALL");
				while(!called_task.pila.empty()) {
					Blocco estratto2=called_task.pop();
					System.out.print("\t");
					System.out.print(estratto2.getNome());
					// trovo la funzione chiamata e la aggiungo all'oggetto Applicazione
					block.addCalledFunction(this.app.getFunction(estratto2.getNome()));
					
				}
			}*/
			System.out.println();
			this.app.substFunction(block);
			System.out.println("+++ Aggiungo la funzione: "+block.getName() );
		}	// if function
		else 
			if (b.getType()==Blocco.CALL) {
			called_task.push(b);}
		else
			st.push(b);
	}
	
}
	
	