// commentata

package statistic;


import general.utils.Utils;

import java.io.PrintWriter;
import java.text.NumberFormat;
import java.util.Vector;

/** Rappresenta le statistiche su un pezzo di codice
 * Attualmente implementa:
 * Minimo, Massimo, Media, Numero di Esecuzioni, Tempo totale
 * @author Pierfrancesco
 *
 */
public class Statistics {

	private static final int MAX_FRAC=3;
	
	private long min;
	private long max;
	private long average;
	private long num_exec;
	private long total_exec;
	
	/** Costruttore di default
	 * 
	 */
	public Statistics() {
		this.min=Long.MAX_VALUE;
		this.max=0;
		this.average=0;
		this.num_exec=0;
		this.total_exec=0;
		
	}
	
	
	/** Calcola le statistiche del vettore passatogli
	 * @param v					vettore contenente i tempi di esecuzione
	 * @param current_timing	tempo di esecuzione corrente
	 */
	public void compute(Vector v, long current_timing) {
		int i=0;
		long min=Long.MAX_VALUE;
		long max=0;
		long avg=0;
		long num;
		long sum;
		long current=0;
		
		// questo per il tempo corrente
		sum = current_timing;
		if (current_timing<min)
			min=current_timing;
		if (current_timing>max)
			max=current_timing;
		
		if (current_timing==0)
			num=0;
		else
			num=1;
		
		// questo per i tempi nel vettore
		for(i=0;i<v.size();i++) {
			// prendo l'i-esimo elemento
			Object current_value=new Object();
			
			current_value=v.get(i);
			current= ((Long) current_value).longValue();
			if (current<min)
				min=current;
			if (current>max)
				max=current;
			sum +=current;
			num++;
		
		}
		if (num>0)
			avg = sum/num;
		
		// aggiorno i membri
		if (min==Long.MAX_VALUE)
			this.min=0;
		else
			this.min=min;
		this.max=max;
		this.average=avg;
		this.num_exec=num;
		this.total_exec=sum;
	}

	/**
	 * @return Returns the average.
	 */
	public long getAverage() {
		return average;
	}

	/**
	 * @return Returns the max.
	 */
	public long getMax() {
		return max;
	}

	/**
	 * @return Returns the min.
	 */
	public long getMin() {
		if (this.min==Long.MAX_VALUE)
			return 0;
		else
			return min;
	}

	/**
	 * @return Returns the num_exec.
	 */
	public long getNum_exec() {
		return num_exec;
	}
	
	/**
	 * @return Returns the total_exec.
	 */
	public long getTotal_exec() {
		return total_exec;
	}

	/** Stampa un piccolo report sulle statistiche del vettore passato
	 * 
	 */
	public void printReport() {
		float max_us, min_us, avg_us, total_us;
		Long max=new Long(this.max);
		Long min=new Long(this.min);
		Long avg=new Long(this.average);
		Long total=new Long(this.total_exec);
		
		max_us=max.floatValue()/ (float) 12800;
		min_us=min.floatValue()/(float) 12800;
		avg_us=avg.floatValue()/(float) 12800;
		total_us=total.floatValue()/(float)12800;
		
		NumberFormat nf=NumberFormat.getNumberInstance();
		nf.setMaximumFractionDigits(MAX_FRAC);
		
		System.out.println("\t    Max: "+nf.format(max_us)+"us");
		System.out.println("\t    Min: "+nf.format(min_us)+"us");
		System.out.println("\t    Average: "+nf.format(avg_us)+"us");
		System.out.println("\t    Num. exec: "+this.num_exec);
		System.out.println("\t    Total exec: "+nf.format(total_us)+"us");
		
	}
	
	/** Genera codice XML delle statistiche
	 * @param n		tabs di identazione
	 */
	public void xml(int n, PrintWriter file) {
		String t=Utils.tabs(n);
		float max_us, min_us, avg_us, total_us;
		Long max=new Long(this.max);
		Long min=new Long(this.min);
		Long avg=new Long(this.average);
		Long total=new Long(this.total_exec);
		max_us=max.floatValue()/ (float) 12800;
		min_us=min.floatValue()/(float) 12800;
		avg_us=avg.floatValue()/(float) 12800;
		total_us=total.floatValue()/(float)12800;
		
		NumberFormat nf=NumberFormat.getNumberInstance();
		nf.setMaximumFractionDigits(MAX_FRAC);
		
		file.println(t+"<STATISTICS>");
		file.println(t+"\t<MAX_VALUE>"+nf.format(max_us)+"us</MAX_VALUE>");
		file.println(t+"\t<MIN_VALUE>"+nf.format(min_us)+"us</MIN_VALUE>");
		file.println(t+"\t<AVG_VALUE>"+nf.format(avg_us)+"us</AVG_VALUE>");
		file.println(t+"\t<TOTAL_EXEC>"+nf.format(total_us)+"us</TOTAL_EXEC>");
		file.println(t+"</STATISTICS>");
		
	}


	public void dtd(PrintWriter file) {
		file.println("<!ELEMENT Stat (Max_v, Min_v, Avg_v,Total_ex)");
		file.println("<!ELEMENT Max_v (#PCDATA)");
		file.println("<!ELEMENT Min_v (#PCDATA)");
		file.println("<!ELEMENT Avg_v (#PCDATA)");
		file.println("<!ELEMENT Total_ex (#PCDATA)");
		
		
	}
	
}
